#!/usr/bin/env python
"""
"""
import unittest
import nose
from GenotypeNetwork import *

import numpy
#testcases = {
class SimpleNetwork(unittest.TestCase):
    """
    Basic test case representing a simple network, composed of 5 genotypes on 3 SNPs.

    All the other test cases are derived from this case. 
    The attributes name, chromosome_len, genotypes, individuals, individuals_dict are used to build the network. 
    All the other attributes that have a name beginning with "expected_" are used for testing. They represent the expected values that the network and its subgraphs should have, and they should be calculated by hand. If some of these attributes don't match, the testing suite will return an error.

    """
    # Network definition
    name = "simple_network"
    chromosome_len = 3
#                   EUR    EUR    AFR    AFR    AFR
    genotypes =   ['001', '000', '000', '000', '010']
    individuals = ['HG1', 'HG2', 'HG3', 'HG4', 'HG5']
    individuals_dict = {
                    'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                    'HG2': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'bad'}, 
                    'HG3': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                    'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                    'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
    }

    # Expected values
    expected_str_by_pop = {
                    'glob' : "Genotype Network (name = 'simple_network', chromosome lenght = 3, |V| = 3, |E| = 2)",
                    "AMR": "Genotype Network (name = 'simple_network_AMR', chromosome lenght = 3, |V| = 0, |E| = 0)",
                    "ASN": "Genotype Network (name = 'simple_network_ASN', chromosome lenght = 3, |V| = 0, |E| = 0)",
                    "EUR": "Genotype Network (name = 'simple_network_EUR', chromosome lenght = 3, |V| = 2, |E| = 1)",
                    "AFR": "Genotype Network (name = 'simple_network_AFR', chromosome lenght = 3, |V| = 2, |E| = 1)",
    }
    expected_report_by_pop = {
                    'glob': "simple_network glob 3 5 3 2 1 2 0.6667 1.6667 1.3333 1.3333 2 0.3333 0.7778",
                    'AMR': "simple_network_AMR AMR 3 0 0 0 0 0 nan nan nan nan 0 nan nan",
                    'ASN': "simple_network_ASN ASN 3 0 0 0 0 0 nan nan nan nan 0 nan nan",
                    'EUR': "simple_network_EUR EUR 3 2 2 1 1 1 1.0 1.0 1.0 1.0 1 0.0 1.0",
                    'AFR': "simple_network_AFR AFR 3 3 2 1 1 1 1.0 1.5 1.0 1.0 1 0.0 1.0"
    }
    expected_nodes_by_pop = {
                    'glob': ['001', '000', '010'],
                    'AMR': [],
                    'ASN': [],
                    'EUR': ['001', '000'],
                    'AFR': ['000', '010']
    }

    expected_individuals_by_pop = {
                    'glob': [['HG1'], ['HG2', 'HG3', 'HG4'], ['HG5']],
                    'AMR': [],
                    'ASN': [],
                    'EUR': [['HG1'], ['HG2']],
                    'AFR': [['HG3', 'HG4'], ['HG5']]
    }

    expected_n_datapoints_by_pop = {
                    'glob': [1,3,1],
                    'AMR': [],
                    'ASN': [],
                    'EUR': [1, 1],
                    'AFR': [2, 1]
    }
    expected_diameter_by_pop = {
                    'glob': 2,
                    'AMR': 0,
                    'ASN': 0,
                    'EUR': 1,
                    'AFR': 1
    }
    expected_avdegree_by_pop = {
                    'glob': round((1 + 2 + 1) / 3., 4),
                    'AMR': numpy.nan,
                    'ASN': numpy.nan,
                    'EUR': (1*1 + 1*1) / 2.,
                    'AFR': (1*1 + 1*1) / 2.
    }
    expected_av_w_degree_by_pop = {
                    'glob': round(((1./5)*1 + (2./5)*3 + (1./5)*1) / 3., 4),
                    'AMR': numpy.nan,
                    'ASN': numpy.nan,
#                    'EUR': (1./5) + (1./5),
                    'EUR': 1/2.,
                    'AFR': ((2./3)*1 + (1./3)*1) / 2.
    }

    def setUp(self): # TODO: see if you can use setUpAll
        self.network = GenotypeNetwork(self.chromosome_len, name = self.name)
        self.network.populate_from_binary_strings_and_individuals(genotypes=self.genotypes, individuals=self.individuals, 
                                individuals_dict=self.individuals_dict)
        self.network.__calculate_network_properties__()
        self.subgraphs = {}
        self.pops = ("AMR", "ASN", "EUR", "AFR")
        for pop in self.pops:
            self.subgraphs[pop] = self.network.subgraph_by_continent(pop)
            self.subgraphs[pop].__calculate_network_properties__()

    def test_NodesNames(self):
        self.assertEquals(sorted(self.network.vs['genotype']), sorted(self.expected_nodes_by_pop['glob']))
        for pop in self.pops:
            print self.subgraphs[pop]
            self.assertEquals(sorted(self.subgraphs[pop].vs['genotype']), sorted(self.expected_nodes_by_pop[pop]))

    def test_NodesDensities(self):
        self.assertEquals(self.network.vs['n_datapoints'], self.expected_n_datapoints_by_pop['glob'])
        for pop in self.pops:
            print pop, self.subgraphs[pop].vs['n_datapoints']
            self.assertEquals(sorted(self.subgraphs[pop].vs['n_datapoints']), sorted(self.expected_n_datapoints_by_pop[pop]))

    def test_NodesIndividuals(self):
        print self.network['name']
        for node in self.network.vs:
            print node['genotype']
            self.assertEquals(sorted(node['node_individuals']), sorted(self.expected_individuals_by_pop['glob'][node.index]))
        for pop in self.pops:
            print pop, self.subgraphs[pop].vs['node_individuals']
            for node in self.subgraphs[pop].vs:
                print node.index
                print node['node_individuals']
                self.assertEquals(sorted(node['node_individuals']), sorted(self.expected_individuals_by_pop[pop][node.index]))

    def test_DiameterByPop(self):
        """check that the diameter of each subgraph is calculated correctly.
        """
        print self.network.diameter()
        print self.network['name']
        self.assertEquals(self.network.diameter(), self.expected_diameter_by_pop['glob'])
        for pop in self.pops:
            print pop, self.subgraphs[pop].diameter(), self.expected_diameter_by_pop[pop] 
            nose.tools.assert_almost_equals(self.subgraphs[pop].diameter(), self.expected_diameter_by_pop[pop], places=10) 

    def test_AvDegreeByPop(self):
        """The Average Degree is the average of the degree of all the nodes, divided by the number of nodes
        """
        print self.network['name']
        print "global av_degree:", self.network['av_degree']
        print self.network['name']
        self.assertEquals(self.network['av_degree'], self.expected_avdegree_by_pop['glob'])
        for pop in self.pops:
            expected = self.expected_avdegree_by_pop[pop]
            observed = self.subgraphs[pop]['av_degree']
            print self.network['name'], pop, expected, observed
            if expected is numpy.nan:
                self.assertEquals(numpy.nan_to_num(expected), numpy.nan_to_num(observed))
            else:
                nose.tools.assert_almost_equals(expected, observed)


    def test_AvWeightedDegreeByPop(self):
        """The Average Degree is the average of the degree of all the nodes, divided by the number of nodes, multiplied by their frequency
        """
        print "global av_w_degree:", self.network['av_w_degree']
        print self.network['name']
        self.assertEquals(self.network['av_w_degree'], self.expected_av_w_degree_by_pop['glob'])
        for pop in self.pops:
            expected = self.expected_av_w_degree_by_pop[pop]
            observed = self.subgraphs[pop]['av_w_degree']
            print self.network['name'], pop, expected, observed
            if expected is numpy.nan:
                self.assertEquals(numpy.nan_to_num(expected), numpy.nan_to_num(observed))
            else:
                nose.tools.assert_almost_equals(expected, observed)


    def test_CSVReportByPop(self):
        print self.network.csv_report()
        for pop in self.pops:
            print self.subgraphs[pop].csv_report()
#            nose.tools.assert_sequence_equal( self.subgraphs[pop].csv_report(), self.expected_report_by_pop[pop])
            nose.tools.assert_equals( self.subgraphs[pop].csv_report(), self.expected_report_by_pop[pop])

    def test_CSVReportGlob(self):
        print self.network.csv_report()
#        nose.tools.assert_sequence_equal( self.network.csv_report(), self.expected_report_by_pop['glob'])
        nose.tools.assert_equals( self.network.csv_report(), self.expected_report_by_pop['glob'])

    def test_StrGlob(self):
        s = self.network.__str__()
        print s
#        nose.tools.assert_sequence_equal(s, self.expected_str_by_pop['glob'])
        nose.tools.assert_equals(s, self.expected_str_by_pop['glob'])

    def test_StrByPop(self):
        for pop in self.pops:
            observed = self.subgraphs[pop].__str__()
            expected = self.expected_str_by_pop[pop]
            print pop, observed, expected
#            nose.tools.assert_sequence_equal(observed, expected)
            nose.tools.assert_equals(observed, expected)
#        assert (1==2)

    def test_NObservations(self):
        """the sum of subpop node n_datapoints should be equal to the n_datapoints of the parent graph"""
#        parent_n_datapoints = sum(self.network.vs['n_datapoints'])
#        print parent_n_datapoints
        print [ind for ind in self.network['individuals'] if self.network['individuals_dict'][ind.replace('_1', '').replace('_2', '')]['continent'] in self.pops]
        parent_n_datapoints = len([ind for ind in self.network['individuals'] if self.network['individuals_dict'][ind.replace('_1', '').replace('_2', '')]['continent'] in self.pops])
#        pops_n_datapoints = sum([sum(pop_network['n_datapoints']) for pop_network in self.subgraphs[pop] for pop in self.pops]) 
        pops_n_datapoints = sum([sum(self.subgraphs[pop].vs['n_datapoints']) for pop in self.pops])
#        print pops_n_datapoints
#        for pop in self.pops:
#            print pop,
#            for genotype in self.subgraphs[pop].vs:
#                print len(genotype['node_individuals']), genotype['n_datapoints'],
#            print self.subgraphs[pop].vs['n_datapoints']
#        print [len(self.subgraphs[pop].vs['node_individuals']) for pop in self.pops]
        self.assertEquals(parent_n_datapoints, pops_n_datapoints)

class DiploidNetwork(SimpleNetwork):
#    self.network = GenotypeNetwork(3, 'sample_phenotype')
    name = "diploid_network"
    chromosome_len = 3
#                   EUR       EUR      EUR      EUR      AFR      AFR      AFR      AFR      AFR      AFR
    genotypes =   ['001',    '000',   '000',   '000',   '010',   '100',   '101',   '111',   '000',   '101']
    individuals = ['HG1_1', 'HG1_2', 'HG2_1', 'HG2_2', 'HG3_1', 'HG3_2', 'HG4_1', 'HG4_2', 'HG5_1', 'HG5_2']
    individuals_dict = {
                    'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                    'HG2': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'bad'}, 
                    'HG3': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                    'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                    'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
    }

    expected_str_by_pop = {
                    'glob' : "Genotype Network (name = 'diploid_network', chromosome lenght = 3, |V| = 6, |E| = 6)",
                    "AMR": "Genotype Network (name = 'diploid_network_AMR', chromosome lenght = 3, |V| = 0, |E| = 0)",
                    "ASN": "Genotype Network (name = 'diploid_network_ASN', chromosome lenght = 3, |V| = 0, |E| = 0)",
                    "EUR": "Genotype Network (name = 'diploid_network_EUR', chromosome lenght = 3, |V| = 2, |E| = 1)",
                    "AFR": "Genotype Network (name = 'diploid_network_AFR', chromosome lenght = 3, |V| = 5, |E| = 4)",
    }
    expected_report_by_pop = {
                    'glob': "diploid_network glob 3 10 6 6 1 4 0.4 1.6667 1.8667 2.0 3 2.1667 0.5556",
                    'AMR': "diploid_network_AMR AMR 3 0 0 0 0 0 nan nan nan nan 0 nan nan",
                    'ASN': "diploid_network_ASN ASN 3 0 0 0 0 0 nan nan nan nan 0 nan nan",
                    'EUR': "diploid_network_EUR EUR 3 4 2 1 1 1 1.0 2.0 1.0 1.0 1 0.0 1.0",
                    'AFR': "diploid_network_AFR AFR 3 6 5 4 1 4 0.4 1.2 2.0 1.6 2 2.0 0.5219"
    }
    expected_nodes_by_pop = {
                    'glob': ['001', '000', '010', '100', '101', '111'],
                    'AMR': [],
                    'ASN': [],
                    'EUR': ['000', '001'],
                    'AFR': ['000', '010', '100', '101', '111']
    }

    expected_individuals_by_pop = {
                    'glob': [['HG1_1'], ['HG1_2', 'HG2_1', 'HG2_2', 'HG5_1'], ['HG3_1'], ['HG3_2'], ['HG4_1', 'HG5_2'], ['HG4_2']], 
                    'AMR': [],
                    'ASN': [],
                    'EUR': [['HG1_1'], ['HG2_2', 'HG2_1', 'HG1_2']],
                    'AFR': [['HG5_1'], ['HG3_1'], ['HG3_2'], ['HG4_1', 'HG5_2'], ['HG4_2']]
    }

    expected_n_datapoints_by_pop = {
                    'glob': [1, 4, 1, 1, 2, 1],
                    'AMR': [],
                    'ASN': [],
                    'EUR': [1, 3],
                    'AFR': [1, 1, 1, 2, 1]
    }

    expected_diameter_by_pop = {
                    'glob': 4,
                    'AMR': 0,
                    'ASN': 0,
                    'EUR': 1,
                    'AFR': 4
    }
    expected_avdegree_by_pop = {
                    'glob': round((2 + 3 + 1 + 2 + 3 + 1)/6., 4),
                    'AMR': numpy.nan,
                    'ASN': numpy.nan,
                    'EUR': (1 + 1)/2.,
                    'AFR': (2 + 1 + 2 + 2 + 1)/5.
    }
    expected_av_w_degree_by_pop = {
                    'glob': round(((1./10)*2 + (4./10)*3 + (1./10)*1 + (1./10)*2 + (2./10)*3 + (1./10)*1)/6., 4),
                    'AMR': numpy.nan,
                    'ASN': numpy.nan,
                    'EUR': ((1./4)*1 + (3./4)*1)/2.,
                    'AFR': round(((1./6)*2 + (1./6)*1 + (1./6)*2 + (2./6)*2 + (1./6)*1)/5., 4)
    }

class ChromosomeLen7(SimpleNetwork):
#    self.network = GenotypeNetwork(3, 'sample_phenotype')
    name = "chromosome_len_7"
    chromosome_len = 7
#                   EUR           EUR       ASN        AFR         AFR
    genotypes =   ['0010001', '0000110', '0100000', '0000000', '0100000']
    individuals = ['HG1', 'HG2', 'HG3', 'HG4', 'HG5']
    individuals_dict = {
                    'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                    'HG2': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'bad'}, 
                    'HG3': {'continent': 'ASN', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                    'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                    'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
    }

    expected_str_by_pop = {
                    'glob' : "Genotype Network (name = 'chromosome_len_7', chromosome lenght = 7, |V| = 4, |E| = 1)",
                    "AMR": "Genotype Network (name = 'chromosome_len_7_AMR', chromosome lenght = 7, |V| = 0, |E| = 0)",
                    "ASN": "Genotype Network (name = 'chromosome_len_7_ASN', chromosome lenght = 7, |V| = 1, |E| = 0)",
                    "EUR": "Genotype Network (name = 'chromosome_len_7_EUR', chromosome lenght = 7, |V| = 2, |E| = 0)",
                    "AFR": "Genotype Network (name = 'chromosome_len_7_AFR', chromosome lenght = 7, |V| = 2, |E| = 1)",
    }
    expected_report_by_pop = {
                    'glob': "chromosome_len_7 glob 7 5 4 1 3 1 0.1667 1.25 1.0 0.5 1 0.0 0.2917",
                    'AMR': "chromosome_len_7_AMR AMR 7 0 0 0 0 0 nan nan nan nan 0 nan nan",
                    'ASN': "chromosome_len_7_ASN ASN 7 1 1 0 1 0 nan 1.0 nan 0.0 0 0.0 nan",
                    'EUR': "chromosome_len_7_EUR EUR 7 2 2 0 2 0 0.0 1.0 nan 0.0 0 0.0 0.5",
                    'AFR': "chromosome_len_7_AFR AFR 7 2 2 1 1 1 1.0 1.0 1.0 1.0 1 0.0 1.0"
    }
    expected_nodes_by_pop = {
#                    'glob': ['0000000', '0000110', '0010001', '0100000'],
                    'glob': ['0010001', '0000110', '0100000', '0000000'],
                    'AMR': [],
                    'ASN': ['0100000'],
                    'EUR': ['0000110', '0010001'],
                    'AFR': ['0000000', '0100000']
    }

    expected_individuals_by_pop = {
                    'glob': [['HG1'], ['HG2'], ['HG3', 'HG5'], ['HG4']],
                    'AMR': [],
                    'ASN': [['HG3']],
                    'EUR': [['HG1'], ['HG2']],
                    'AFR': [['HG5'], ['HG4']]
    }

    expected_n_datapoints_by_pop = {
                    'glob': [1, 1, 2, 1],
                    'AMR': [],
                    'ASN': [1],
                    'EUR': [1, 1],
                    'AFR': [1, 1]
    }

    expected_diameter_by_pop = {
                    'glob': 1,
                    'AMR': 0,
                    'ASN': 0,
                    'EUR': 0,
                    'AFR': 1
    }
    expected_avdegree_by_pop = {
                    'glob': round((0 + 0 + 1 + 1)/4., 4),
                    'AMR': numpy.nan,
                    'ASN': 0,
                    'EUR': 0,
                    'AFR': 1
    }
    expected_av_w_degree_by_pop = {
                    'glob': round((0 + 0 + (2./5)*1 + (1./5)*1)/4., 4),
                    'AMR': numpy.nan,
                    'ASN': 0,
                    'EUR': 0,
                    'AFR': 1/2.
    }

class OneNode10Individuals(SimpleNetwork):
    name = 'onenode_10inds'
#                    EUR    EUR    AFR    AFR    AFR    AFR    AMR    EUR    AFR    ASN
    genotypes  =   ['000', '000', '000', '000', '000', '000', '000', '000', '000', '000']
    individuals =  ['HG1', 'HG2', 'HG3', 'HG4', 'HG5', 'HG6', 'HG7', 'HG8', 'HG9', 'HG10']
    individuals_dict = {
                     'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                     'HG2': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'bad'}, 
                     'HG3': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                     'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                     'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                     'HG6': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                     'HG7': {'continent': 'AMR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                     'HG8': {'continent': 'EUR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                     'HG9': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                     'HG10': {'continent': 'ASN', 'subpop': 'YRI', 'phenotype1': 'good'}, 
    }
    expected_str_by_pop = {
                    'glob' : "Genotype Network (name = 'onenode_10inds', chromosome lenght = 3, |V| = 1, |E| = 0)",
                    "AMR": "Genotype Network (name = 'onenode_10inds_AMR', chromosome lenght = 3, |V| = 1, |E| = 0)",
                    "ASN": "Genotype Network (name = 'onenode_10inds_ASN', chromosome lenght = 3, |V| = 1, |E| = 0)",
                    "EUR": "Genotype Network (name = 'onenode_10inds_EUR', chromosome lenght = 3, |V| = 1, |E| = 0)",
                    "AFR": "Genotype Network (name = 'onenode_10inds_AFR', chromosome lenght = 3, |V| = 1, |E| = 0)",
    }
    expected_report_by_pop = {
                    'glob': "onenode_10inds glob 3 10 1 0 1 0 nan 10.0 nan 0.0 0 0.0 nan",
                    'AMR': "onenode_10inds_AMR AMR 3 1 1 0 1 0 nan 1.0 nan 0.0 0 0.0 nan",
                    'ASN': "onenode_10inds_ASN ASN 3 1 1 0 1 0 nan 1.0 nan 0.0 0 0.0 nan",
                    'EUR': "onenode_10inds_EUR EUR 3 3 1 0 1 0 nan 3.0 nan 0.0 0 0.0 nan",
                    'AFR': "onenode_10inds_AFR AFR 3 5 1 0 1 0 nan 5.0 nan 0.0 0 0.0 nan"
    }
    expected_nodes_by_pop = {
                     'glob': ['000'],
                     'AMR': ['000'],
                     'ASN': ['000'],
                     'EUR': ['000'],
                     'AFR': ['000']
    }
    expected_individuals_by_pop = {
                    'glob': ['HG1 HG2 HG3 HG4 HG5 HG6 HG7 HG8 HG9 HG10'.split(' ')],
                    'AMR': [['HG7']],
                    'ASN': [['HG10']],
                    'EUR': [['HG1', 'HG2', 'HG8']],
                    'AFR': [['HG3', 'HG4', 'HG5', 'HG6', 'HG9']]
    }
    expected_n_datapoints_by_pop = {
                    'glob': [10],
                    'AMR': [1],
                    'ASN': [1],
                    'EUR': [3],
                    'AFR': [5]
    }
    expected_diameter_by_pop = {
                    'glob': 0,
                    'AMR': 0,
                    'ASN': 0,
                    'EUR': 0,
                    'AFR': 0
    }

    expected_avdegree_by_pop = {
                    'glob': 0,
                    'AMR': 0,
                    'ASN': 0,
                    'EUR': 0,
                    'AFR': 0
    }
    expected_av_w_degree_by_pop = {
                    'glob': 0,
                    'AMR': 0,
                    'ASN': 0,
                    'EUR': 0,
                    'AFR': 0
    }



class Simple_5SNPs_10Inds(SimpleNetwork):
    name = 'simple_5SNPs_10Inds'
#                    EUR    EUR    AFR    AFR    AFR    AFR    AMR    EUR    AFR    ASN
    genotypes  =   ['001', '000', '000', '000', '010', '000', '100', '111', '110', '101']
    individuals =  ['HG1', 'HG2', 'HG3', 'HG4', 'HG5', 'HG6', 'HG7', 'HG8', 'HG9', 'HG10']
    individuals_dict = {
                     'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                     'HG2': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'bad'}, 
                     'HG3': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                     'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                     'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                     'HG6': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                     'HG7': {'continent': 'AMR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                     'HG8': {'continent': 'EUR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                     'HG9': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                     'HG10': {'continent': 'ASN', 'subpop': 'YRI', 'phenotype1': 'good'}, 
    }
    expected_str_by_pop = {
                    'glob' : "Genotype Network (name = 'simple_5SNPs_10Inds', chromosome lenght = 3, |V| = 7, |E| = 9)", 
                    "AMR": "Genotype Network (name = 'simple_5SNPs_10Inds_AMR', chromosome lenght = 3, |V| = 1, |E| = 0)",
                    "ASN": "Genotype Network (name = 'simple_5SNPs_10Inds_ASN', chromosome lenght = 3, |V| = 1, |E| = 0)",
                    "EUR": "Genotype Network (name = 'simple_5SNPs_10Inds_EUR', chromosome lenght = 3, |V| = 3, |E| = 1)",
                    "AFR": "Genotype Network (name = 'simple_5SNPs_10Inds_AFR', chromosome lenght = 3, |V| = 3, |E| = 2)",
    }
    expected_report_by_pop = {
                    'glob': "simple_5SNPs_10Inds glob 3 10 7 9 1 3 0.4286 1.4286 1.7143 2.5714 3 2.1429 0.5861",
                    'AMR': "simple_5SNPs_10Inds_AMR AMR 3 1 1 0 1 0 nan 1.0 nan 0.0 0 0.0 nan",
                    'ASN': "simple_5SNPs_10Inds_ASN ASN 3 1 1 0 1 0 nan 1.0 nan 0.0 0 0.0 nan",
                    'EUR': "simple_5SNPs_10Inds_EUR EUR 3 3 3 1 2 1 0.3333 1.0 1.0 0.6667 1 0.0 0.4444",
                    'AFR': "simple_5SNPs_10Inds_AFR AFR 3 5 3 2 1 2 0.6667 1.6667 1.3333 1.3333 2 0.3333 0.7778"
    }
    expected_nodes_by_pop = {
                     'glob': ['001', '000', '010', '100', '111', '110', '101'],
                     'AMR': ['100'],
                     'ASN': ['101'],
                     'EUR': ['001', '000', '111'],
                     'AFR': ['000', '010', '110']
    }
    expected_individuals_by_pop = {
                    'glob': [['HG1'], 'HG2 HG3 HG4 HG6'.split(' '), ['HG5'], ['HG7'], ['HG8'], ['HG9'], ['HG10']],
                    'AMR': [['HG7']],
                    'ASN': [['HG10']],
                    'EUR': [['HG1'], ['HG2'], ['HG8']],
#                    'EUR': [['HG3', 'HG4', 'HG6'], ['HG2'], ['HG8']],
                    'AFR': [['HG3', 'HG4', 'HG6'], ['HG5'], ['HG9']]
    }
    expected_n_datapoints_by_pop = {
                    'glob': [1, 4, 1, 1, 1, 1, 1],
                    'AMR': [1],
                    'ASN': [1],
                    'EUR': [1, 1, 1],
                    'AFR': [3, 1, 1]
    }
    expected_diameter_by_pop = {
                    'glob': 3,
                    'AMR': 0,
                    'ASN': 0,
                    'EUR': 1,
                    'AFR': 2
    }
    expected_avdegree_by_pop = {
                    'glob': round((2 + 3 + 2 + 3 + 2 + 3 + 3)/7., 4),
                    'AMR': numpy.nan,
                    'ASN': numpy.nan,
                    'EUR': round((1 + 1 + 0)/3., 4),
                    'AFR': round((1 + 2 + 1)/3., 4)
    }
    expected_av_w_degree_by_pop = {
                    'glob': round(((1./10)*2 + (4./10)*3 + (1./10)*2 + (1./10)*3 + (1./10)*2 + (1./10)*3 + (1./10)*3)/7., 4),
                    'AMR': 0,
                    'ASN': 0,
                    'EUR': round(((1./3)+(1./3))/3., 4),
                    'AFR': ((3./5)*1 + (1./5)*2 + (1./5)*1)/ 3.
    }




if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
"""
"""
import unittest
import nose
from GenotypeNetwork import GenotypeNetworkInitError, GenotypeNetworkAttributeError

class Simple_5SNPs_Network(unittest.TestCase):
    
    chromosome_len = 5
    name = 'sample_phenotype'
    genotypes = ['00001', '01000', '00000', '01001', '01101']

    # expected values
    expected_summary = "5 nodes, 5 edges, undirected\n\nNumber of components: 1\nDiameter: 3\nDensity: 0.5000\nAverage path length: 1.6000"
#    expected_nodes = ['00000', '00001', '01000', '01001', '01101']
    expected_nodes = ['01101', '00001', '00000', '01000', '01001']
    expected_degree = [1, 2, 2, 2, 3]
    expected_max_closeness = '01001'
    expected_closeness = [0.5, 0.6667, 0.5714, 0.6667, 0.8]

    def setUp(self):
        from GenotypeNetwork import GenotypeNetwork
        self.network = GenotypeNetwork(self.chromosome_len, self.name)
        self.network.populate_from_binary_strings(self.genotypes)
    
    def test_summary(self):
        print "PRINT", self.network.summary()

        output = self.network.summary()
        print output
        self.assertEqual(output, self.expected_summary)

    def test_SimpleDegreeClosenessReport(self):
        tested_attributes = ['genotype', 'degree', 'closeness']
        expected_degree_report = reproduce_valuesBySnp_output(tested_attributes, [self.expected_nodes, self.expected_degree, self.expected_closeness])
        print expected_degree_report
#        nose.tools.assert_sequence_equal(expected_degree_report, self.network.values_by_node(tested_attributes))
        nose.tools.assert_equals(expected_degree_report, self.network.values_by_node(tested_attributes))

    def test_NetworkNameReport(self):
        tested_attributes = ['genotype', 'name']
        self.expected_name = [self.name] * len(self.genotypes)
        expected_degree_report = reproduce_valuesBySnp_output(tested_attributes, [self.expected_nodes, self.expected_name])
        print expected_degree_report
#        nose.tools.assert_sequence_equal(expected_degree_report, self.network.values_by_node(tested_attributes))
        nose.tools.assert_equals(expected_degree_report, self.network.values_by_node(tested_attributes))

    def test_SimpleDegreeReport(self):
        tested_attributes = ['genotype', 'degree']
        expected_degree_report = reproduce_valuesBySnp_output(tested_attributes, [self.expected_nodes, self.expected_degree])
#        nose.tools.assert_sequence_equal(expected_degree_report, self.network.values_by_node(tested_attributes))
        nose.tools.assert_equals(expected_degree_report, self.network.values_by_node(tested_attributes))


    @nose.tools.raises(GenotypeNetworkAttributeError)
    def test_InexistentAttribute(self):
        tested_attributes = ['genotype', 'basdadas']
#        expected_degree_report = reproduce_valuesBySnp_output(tested_attributes, [self.expected_nodes, self.expected_degree])
#        print expected_degree_report
        self.network.values_by_node(tested_attributes)

class Simple_5SNPs_5Inds_Network(Simple_5SNPs_Network):
    
    chromosome_len = 5
    name = 'sample_phenotype'
    genotypes =   ['00001',    '00000',   '00000',   '00001',   '00010',   '00100',   '00001',   '00001',   '00010',   '00001']
    individuals = ['HG1_1', 'HG1_2', 'HG2_1', 'HG2_2', 'HG3_1', 'HG3_2', 'HG4_1', 'HG4_2', 'HG5_1', 'HG5_2']
    individuals_dict = {
                    'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                    'HG2': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'bad'}, 
                    'HG3': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                    'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                    'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
    }


    # expected values
    expected_summary = "4 nodes, 3 edges, undirected\n\nNumber of components: 1\nDiameter: 2\nDensity: 0.5000\nAverage path length: 1.5000"
#    expected_nodes = ['00000', '00001', '01000', '01001', '01101']
    expected_nodes = ['00100', '00001', '00000', '00010']
    expected_degree = [1, 1, 3, 1]
    expected_max_closeness = '00000'
    expected_closeness = [0.6, 0.6, 1.0, 0.6]

    def setUp(self):
        from GenotypeNetwork import GenotypeNetwork
        self.network = GenotypeNetwork(self.chromosome_len, self.name)
        self.network.populate_from_binary_strings(self.genotypes)



def reproduce_valuesBySnp_output(values_names, expected_values):
    """
    Utility function, used for tests only, to reproduce what the output of 
    GenotypeNetwork.values_by_snp should look like, given a list of expected
    values.

    expected_values should be a list of lists of expected values per node.
    Example: 

    >>> reproduce_valuesBySnp_output(values_names=['genotype', 'degree'], 
    ...                 expected_values=['001', '000'], [1, 2]) #doctest: +IGNORE_WHITESPACE
    #genotype   degree
    001 1
    000 2

    See test_reproduceValuesBySnp for example and tests on this function.
    """
    output = '#' + '\t'.join(values_names) + '\n'
    for line_attrs in zip(*expected_values):
        output += '\t'.join([str(att) for att in line_attrs]) + '\n'
    return output[:-1]



def test_reproduceValuesBySnp():
    """
    >>> import GenotypeNetwork
    >>> network = GenotypeNetwork.GenotypeNetwork(3, genotypes = ['001', '000', '000'])
    >>> expected_nodes = ['001', '000']
    >>> [v['genotype'] for v in network.vs] == expected_nodes
    True
    >>> expected_degrees = [1, 1]
    >>> reproduced_output = reproduce_valuesBySnp_output(values_names=['genotype', 'degree'],
    ...             expected_values=[expected_nodes, expected_degrees])  #doctest: +NORMALIZE_WHITESPACE
    >>> print reproduced_output #doctest: +NORMALIZE_WHITESPACE
    #genotype   degree
    001 1
    000 1

    """
#    >>> assert(reproduced_output == "#genotype\tdegree\n001\t1\n000\t1")  #doctest: +NORMALIZE_WHITESPACE
#    True
    pass

def test_GoodParameters():
    """
    Test different types of valid inputs for GenotypeNetwork, and check if they can print a
    values_by_node report, with the default values, without returning errors.
    """
    good_genotypes = [
        (3, ['001', '000', '010'], ""),
        (1, ['1', '0', '0'], ""),
        (4, ['0001', '0010', '1010'], ""),
        (3, ['000', '000', '000'], ""),
        (5, ['00001', '11000', '01110'], ""),
        (3, ['111', '100', '010'], ""),
        ]
    for params_set in good_genotypes:
        yield check_GoodGenotypes, params_set[0], params_set[1], params_set[2]


def check_GoodGenotypes(chromosome_len, genotypes, expectedoutput):
    from GenotypeNetwork import GenotypeNetwork
    net = GenotypeNetwork(chromosome_len, genotypes=genotypes)
    net.values_by_node()

#def test_WrongGenotypes():
#   """
#   Test different types of wrong inputs for GenotypeNetwork, they should all return errors
#   """
#   wrong_genotypes = [
#       (3, ['001', '000', '020'], ""),         # contains a "2"
#       (2, ['001', '000', '010'], ""),         # wrong chromosome_len
#       (3, ['001', '000', '00'], ""),          # wrong length
#       (5, ['00101', '11000', '0000'], ""),    # wrong length
#       (3, '001', ''),                         # genotypes is not a list
#       ]
#
#   for params_set in wrong_genotypes: 
#       yield check_WrongGenotypesError, params_set[0], params_set[1], params_set[2]
#
#@nose.tools.raises(GenotypeNetworkInitError)
#def check_WrongGenotypesError(chromosome_len, genotypes, expectederrormessage):
#   from GenotypeNetwork import GenotypeNetwork
#   print chromosome_len, genotypes
#   net = GenotypeNetwork(chromosome_len, genotypes=genotypes)
#   print genotypes
#

if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
"""
Test the GenotypeNetwork.populate_from_binary_strings and populate_from_binary_strings_and_individuals functions, using different distance values
"""
import unittest
import nose

@nose.plugins.attrib.attr("nocluster")
class Simple_5SNPs_Network(unittest.TestCase):
    """Simple 5 SNPs, 10 individuals network

    This network should have 2 components if distance==1, and 1 if distance==2 or distance==3
    """
    genotypes = ['00000', '00000', '00000', '00000', '11000', '11000', '11000', '11000', '00011', '00011']
    chromosome_len = 5

    expected_distance_1_summary = "3 nodes, 0 edges, undirected\n\nNumber of components: 3\nDiameter: 0\nDensity: 0.0000\nAverage path length: nan"
    expected_distance_2_summary = "3 nodes, 2 edges, undirected\n\nNumber of components: 1\nDiameter: 2\nDensity: 0.6667\nAverage path length: 1.3333"
    expected_distance_3_summary = "3 nodes, 2 edges, undirected\n\nNumber of components: 1\nDiameter: 2\nDensity: 0.6667\nAverage path length: 1.3333"

    expected_distance_1_ncomponents = 3
    expected_distance_2_ncomponents = 1    
    expected_distance_3_ncomponents = 1    

    def setUp(self):
        from GenotypeNetwork import GenotypeNetwork

        self.network_distance_1 = GenotypeNetwork(self.chromosome_len, 'Distance==1 network')
        self.network_distance_1.populate_from_binary_strings(self.genotypes, distance=1)
        
        self.network_distance_2 = GenotypeNetwork(self.chromosome_len, 'Distance==2 network')
        self.network_distance_2.populate_from_binary_strings(self.genotypes, distance=2)

        self.network_distance_3 = GenotypeNetwork(self.chromosome_len, 'Distance==3 network')
        self.network_distance_3.populate_from_binary_strings(self.genotypes, distance=3)

    def test_Distance_1_Summary(self):
        print [vertex['genotype'] for vertex in self.network_distance_1.vs()]
        print [edge.tuple for edge in self.network_distance_1.es()]
#        nose.tools.assert_sequence_equal(self.expected_distance_1_summary, self.network_distance_1.summary())
        nose.tools.assert_equals(self.expected_distance_1_summary, self.network_distance_1.summary())

    def test_Distance_2_Summary(self):
        print [vertex['genotype'] for vertex in self.network_distance_2.vs()]
        print [edge.tuple for edge in self.network_distance_2.es()]
#        nose.tools.assert_sequence_equal(self.expected_distance_2_summary, self.network_distance_2.summary())
        nose.tools.assert_equals(self.expected_distance_2_summary, self.network_distance_2.summary())

    def test_Distance_3_Summary(self):
        print [vertex['genotype'] for vertex in self.network_distance_3.vs()]
        print [edge.tuple for edge in self.network_distance_3.es()]
#        nose.tools.assert_sequence_equal(self.expected_distance_3_summary, self.network_distance_3.summary())
        nose.tools.assert_equals(self.expected_distance_3_summary, self.network_distance_3.summary())

    def test_Distance_1_NComponents(self):
        nose.tools.assert_equal(self.expected_distance_1_ncomponents, len(self.network_distance_1.components()))

    def test_Distance_2_NComponents(self):
        nose.tools.assert_equal(self.expected_distance_2_ncomponents, len(self.network_distance_2.components()))

    def test_Distance_3_NComponents(self):
        nose.tools.assert_equal(self.expected_distance_3_ncomponents, len(self.network_distance_3.components()))


@nose.plugins.attrib.attr("nocluster")
class Simple_OneNode_Network(Simple_5SNPs_Network):
    """
    One node network. All networks should be the same, regardless of the distance attribute.
    """
    genotypes = ['00000', '00000', '00000', '00000', '00000', '00000', '00000', '00000', '00000', '00000']
    chromosome_len = 5

    expected_distance_1_summary = "1 nodes, 0 edges, undirected\n\nNumber of components: 1\nDiameter: 0\nDensity: nan\nAverage path length: nan"
    expected_distance_2_summary = "1 nodes, 0 edges, undirected\n\nNumber of components: 1\nDiameter: 0\nDensity: nan\nAverage path length: nan"
    expected_distance_3_summary = "1 nodes, 0 edges, undirected\n\nNumber of components: 1\nDiameter: 0\nDensity: nan\nAverage path length: nan"

    expected_distance_1_ncomponents = 1
    expected_distance_2_ncomponents = 1    
    expected_distance_3_ncomponents = 1    

@nose.plugins.attrib.attr("nocluster")
class BiggerNetwork1(Simple_5SNPs_Network):
    """
    Three Nodes Network.  This is bigger example than the others and attributes are only checked manually.
    """
    genotypes = ['00001', '10000', '01000', '00010', '00100', '01000', '00010', '11111', '00010', '01100']
    chromosome_len = 5

    expected_distance_1_summary = "7 nodes, 2 edges, undirected\n\nNumber of components: 5\nDiameter: 2\nDensity: 0.0952\nAverage path length: 1.3333"
    expected_distance_2_summary = "7 nodes, 12 edges, undirected\n\nNumber of components: 2\nDiameter: 2\nDensity: 0.5714\nAverage path length: 1.2000"
    expected_distance_3_summary = "7 nodes, 16 edges, undirected\n\nNumber of components: 1\nDiameter: 2\nDensity: 0.7619\nAverage path length: 1.2381"

    expected_distance_1_ncomponents = 5
    expected_distance_2_ncomponents = 2    
    expected_distance_3_ncomponents = 1    

@nose.plugins.attrib.attr("nocluster")
class BiggerRandomNetwork(Simple_5SNPs_Network):
    """
    Network generated randomly. 100 nodes, 10 SNPs

    """
    genotypes = ['00001', '10000', '01000', '00010', '00100', '01000', '00010', '11111', '00010', '01100']
    chromosome_len = 10

    expected_distance_1_summary = "95 nodes, 28 edges, undirected\n\nNumber of components: 67\nDiameter: 7\nDensity: 0.0063\nAverage path length: 2.4267"
    expected_distance_2_summary = "95 nodes, 234 edges, undirected\n\nNumber of components: 2\nDiameter: 8\nDensity: 0.0524\nAverage path length: 3.5228"
    expected_distance_3_summary = "95 nodes, 732 edges, undirected\n\nNumber of components: 1\nDiameter: 4\nDensity: 0.1639\nAverage path length: 2.0858"

    expected_distance_1_ncomponents = 67
    expected_distance_2_ncomponents = 2    
    expected_distance_3_ncomponents = 1    

    def setUp(self):
        from GenotypeNetwork import GenotypeNetwork
        import random
        random.seed(0)
        self.genotypes = [''.join(str(random.choice((0, 1))) for pos in range(10)) for n_individual in range(100)]

        self.network_distance_1 = GenotypeNetwork(self.chromosome_len, 'Distance==1 network')
        self.network_distance_1.populate_from_binary_strings(self.genotypes, distance=1)
        
        self.network_distance_2 = GenotypeNetwork(self.chromosome_len, 'Distance==2 network')
        self.network_distance_2.populate_from_binary_strings(self.genotypes, distance=2)

        self.network_distance_3 = GenotypeNetwork(self.chromosome_len, 'Distance==3 network')
        self.network_distance_3.populate_from_binary_strings(self.genotypes, distance=3)

@nose.plugins.attrib.attr("nocluster")
class SimpleNetwork_Individuals(Simple_5SNPs_Network):
    chromosome_len = 3
#                   EUR    EUR    AFR    AFR    AFR
    genotypes =   ['101', '000', '000', '111', '110']
    individuals = ['HG1', 'HG2', 'HG3', 'HG4', 'HG5']
    individuals_dict = {
                    'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                    'HG2': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'bad'}, 
                    'HG3': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                    'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                    'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
    }
    expected_distance_1_summary = "4 nodes, 2 edges, undirected\n\nNumber of components: 2\nDiameter: 2\nDensity: 0.3333\nAverage path length: 1.3333"
    expected_distance_2_summary = "4 nodes, 5 edges, undirected\n\nNumber of components: 1\nDiameter: 2\nDensity: 0.8333\nAverage path length: 1.1667"
    expected_distance_3_summary = "4 nodes, 6 edges, undirected\n\nNumber of components: 1\nDiameter: 1\nDensity: 1.0000\nAverage path length: 1.0000"

    expected_distance_1_ncomponents = 2
    expected_distance_2_ncomponents = 1    
    expected_distance_3_ncomponents = 1    

    def setUp(self):
        from GenotypeNetwork import GenotypeNetwork

        self.network_distance_1 = GenotypeNetwork(self.chromosome_len, 'Distance==1 network')
        self.network_distance_1.populate_from_binary_strings_and_individuals(self.genotypes, distance=1, individuals=self.individuals, individuals_dict=self.individuals_dict)
        
        self.network_distance_2 = GenotypeNetwork(self.chromosome_len, 'Distance==2 network')
        self.network_distance_2.populate_from_binary_strings_and_individuals(self.genotypes, distance=2, individuals=self.individuals, individuals_dict=self.individuals_dict)

        self.network_distance_3 = GenotypeNetwork(self.chromosome_len, 'Distance==3 network')
        self.network_distance_3.populate_from_binary_strings_and_individuals(self.genotypes, distance=3, individuals=self.individuals, individuals_dict=self.individuals_dict)




if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
"""
Test multiple call to VCF2Networks, using different parameters, and checking if the programs runs correctly or not.

"""
import subprocess
import unittest
import nose

good_examples = {
    'example1':
        # basic call, including all the default options. 
        'python src/vcf2networks.py  -g data/vcf_filtered/MOGS.recode.vcf -c params/default.yaml -i data/individuals/individuals_annotations.txt',

    'example2':
        # using a different window and distance definition
        'python src/vcf2networks.py  -g data/vcf_filtered/MOGS.recode.vcf -c params/default.yaml     -w 8 -d 2 ',

    'example3': 
        # using PHENOTYPE1 as phenotype
        'python src/vcf2networks.py  -g data/vcf_filtered/MOGS.recode.vcf -c params/default.yaml     -w 12 -p PHENOTYPE1',

    'example4': 
        # using PHENOTYPE1, and subsampling
        'python src/vcf2networks.py  -g data/vcf_filtered/MOGS.recode.vcf -c params/default.yaml     -w 12 -p PHENOTYPE1 --sample 100',

    'example5':
        # testing a compressed file
        'python src/vcf2networks.py  -g data/vcf_filtered/CCR5.recode.vcf.gz -c params/default.yaml  -w 11 -p subpop',

    'example6':
        # subsampling multiple times
        'python src/vcf2networks.py  -g data/vcf_filtered/MOGS.recode.vcf -c params/default.yaml     -w 10 --sample 100 -r 2',

#    'example_fromotherfolder':
#        # calling the script from another folder
#        '(cd ..; python vcf2networks/src/vcf2networks.py  -g vcf2networks/data/vcf_filtered/MOGS.recode.vcf  -c vcf2networks/params/default.yaml -w 11 -s 3 --individuals 100 -p PHENOTYPE1 --individuals_file vcf2networks/data/individuals/individuals_annotations.txt )'
        }


def test_goodcalls():
    for (example_id, example_command) in good_examples.items():
        yield check_goodexample, example_command

def check_goodexample(example_command):
#    print example_command.split()
    print example_command
    subprocess.check_output(example_command.split())

def test_goodcalls_fromotherfolder():
    subprocess.check_output('python vcf2networks/src/vcf2networks.py  -g vcf2networks/data/vcf_filtered/MOGS.recode.vcf  -c vcf2networks/params/default.yaml -w 11 -s 3 --individuals 100 -p PHENOTYPE1 --individuals_file vcf2networks/data/individuals/individuals_annotations.txt'.split(), cwd='..')



#
##class VCFNotSpecified(VCFExistsButInvalid):


bad_examples = {
    'vcffile_exists_but_invalid':
        {
            'cmd': 'python src/vcf2networks.py  -c params/default.yaml  --vcf README.rst   -w 8 -d 2',
            'expected_error': 'Wrong number of columns in the VCF file.'
        },
            
            

    'vcffile_not_specified':
        {
            'cmd': 'python src/vcf2networks.py  -c params/default.yaml  -w 8 -d 2',
            'expected_error' : 'no VCF file provided',
        },

    'vcffile_not_exists':
        {
            'cmd': 'python src/vcf2networks.py  -c params/default.yaml  --vcf sda   -w 8 -d 2',
            'expected_error': 'I can not parse the VCF file provided (sda).'
        },

    'invalid_individuals_file':
        {
            'cmd': 'python src/vcf2networks.py  -c params/default.yaml  --vcf data/vcf_filtered/MOGS.recode.vcf --individuals dasd -w 8 -d 2',
            'expected_error': 'I can not parse the VCF file provided (sda).'
        },
    }

def test_badcalls():
    for (example_id, example_details) in bad_examples.items():
        yield check_badexample, example_details
#
def check_badexample(example_details):
#    print example_details.split()
    print example_details
    with nose.tools.assert_raises(subprocess.CalledProcessError):
        output = subprocess.check_output(example_details['cmd'].split(), stderr=subprocess.STDOUT)
        nose.tools.assert_contains(example_details['expected_error'], output)

#    except:
##        pass












if __name__ == '__main__':
    pass




#!/usr/bin/env python
"""
Parse a file in .binary format and produce a Genotype Network.

When invoked from the command line, it also prints a linear report of network properties.

"""
import logging
import optparse
import os
from custom_exceptions import *
from filteredvcf_to_binary import parse_individuals_file
#import cPickle
import operator 
import re
import random
#import gc
import numpy
from GenotypeNetwork import GenotypeNetwork


def get_options():
    """Get command line arguments
    """
    parser = optparse.OptionParser(usage="python %prog -f genotypes.binary", 
            description=__doc__)
    parser.add_option('-f', '-g', '-v', '--genotypes', dest='genotypes_file', 
            help='File containing the genotypes in binary format, produced \
by src/parse_genotypes.py', default=False)
    parser.add_option('-u', '--no-debug', dest='debug', action='store_false', 
            default=True)
    parser.add_option('-n', '-d', '--distance', dest='distance', type='int',
            help='Distance between nodes in the network. Two nodes will be \
considered being neighbors if they have at least _distance_ \
number of differences',
            default=1)
    parser.add_option('-s', '--upstream', dest='upstream', type='int',
            help='upstream position, to extract a subset of SNPs. \
The SNP at upstream position is included.',
            default=None)
    parser.add_option('-e', '--downstream', dest='downstream', type='int',
            help='downstream position, to extract a subset of SNPs',
            default=None)
    parser.add_option('-i', '--n_individuals', dest='n_individuals', type='int',
            help='Number of individuals to use. If 0, all individuals are used',
            default=0)
    parser.add_option('--hdf5', '--h5',
            help='Use HDF5 internal storage. More memory efficient, but requires the h5py library to be installed.',
            dest='use_hdf5', action='store_true', default=False)
    (options, args) = parser.parse_args()
    
    if options.debug is True:
        logging.basicConfig(level=logging.DEBUG, 
                filename='logs/generate_networks.log')

    if (options.genotypes_file is False) and (len(args) == 0):
        parser.print_help()
        parser.error('generate_networks.py error: no genotypes file provided')
    try:
        binary_file_path = ''
        if options.genotypes_file is not False:
            binary_file_path = options.genotypes_file
        elif args != '':
            binary_file_path = args[0]
        if os.stat(binary_file_path).st_size == 0:
            parser.error("file {0} is empty".format(binary_file_path))
    except:
        print __doc__
        parser.error("Can not open genotypes file {0}".format(options.genotype_file))

    if options.upstream < options.downstream:
        parser.error("Window Start can not be higher than Window End ({0}, {1})".format(options.upstream, options.downstream))

    if options.use_hdf5 is True:
        import h5py

    return (binary_file_path, options)

def parse_hdf5_file(hdf5_file, individuals_dict=None):
    """
    Parse a HDF5 file, and return a numpy matrix
    """
#    raise NotImplemented
    import h5py 
    logging.debug( hdf5_file)
    import os
    logging.debug(os.getcwd())
    import glob
#    logging.debug(glob.glob(hdf5_file))

    if isinstance(hdf5_file, str):      #TODO: this doesn't seem to work
        # hdf5_file is a path to a file. Open it.
        hdf5_file = h5py.File(hdf5_file, 'r')

    if hasattr(hdf5_file, "name"):
        hdf5file_path = hdf5_file.name
        # TODO: avoid closing/reopening of HDF5 files
        hdf5_file.close()
        hdf5_file = h5py.File(hdf5file_path, 'r')
#        gene_name = hdf5file_path.split('/')[-1].split('.',1)[0]
    else:
        hdf5file_path = 'stringIO object'
#        gene_name = 'unknown_gene'
#    print hdf5_file.values()
    gene_name = hdf5_file['indv_annotations'][0,0]
    individuals = []
    filteredout_genotypes = set([])
    #TODO: make sure that this works correctly when individuals_dict is passed as a parameter to this function
#    if individuals_dict is None or individuals_dict is False:   
    indv_annotations = hdf5_file['indv_annotations']
    for indv in indv_annotations:
        individual_key = indv[1].replace("_1", "").replace("_2", "").replace("_a", "").replace("_b", "")
        individuals.append(indv[1])
        individuals_dict.setdefault(individual_key, {})
        individuals_dict[individual_key]['subpop'] = indv[2]
#            continent = indv[3]
        individuals_dict[individual_key]['pop'] = indv[3]
        individuals_dict[individual_key]['continent'] = indv[3]
        individuals_dict[individual_key].setdefault('techniques', '')
#            sex = indv[4]
        individuals_dict[individual_key]['sex'] = 'X'
#    else:
#        individuals = individuals_dict.keys()

    all_genotypes = hdf5_file['genotype_line']
    if len(all_genotypes) == 0:
        raise GenotypeNetworkInitError("No genotypes in {0}".format(hdf5file_path))
#    print individuals
#    print individuals_dict.items()[0:2]

    # FILTER genotypes that are not 0 or 1 
    # NOTE: FILTERING of wrong genotypes is not implemented yet. This is fine
    # because none of the files should contain wrong genotypes.
    filteredout_genotypes = set([])
    for snp_column in range(all_genotypes.len()):
        if max(all_genotypes[snp_column, :]) > 1:
            filteredout_genotypes.update(snp_column)
            
    if len(filteredout_genotypes) > 0:
        kept = [n for n in range(total_number_snps) if n not in filteredout_genotypes]
        logging.warn("Triallelic SNP found in {0}".format(binaryfile_path))
        raise NotImplemented("filtering of Triallelic SNPs is not implemented yet")
    # MAF FILTER: not implemented at this stage. MAF filters are applied when filtering the VCF file.


    return all_genotypes, individuals, individuals_dict, gene_name


def parse_binary_file(binary_file, individuals_dict=None):
    """
    Parse a binary file, and return a numpy matrix
    """
    if isinstance(binary_file, str):
        # binary_file is a path to a file. Open it.
        binary_file = open(binary_file, 'r')

    if hasattr(binary_file, "name"):
        binaryfile_path = binary_file.name
#        gene_name = binaryfile_path.split('/')[-1].split('.',1)[0]
    else:
        binaryfile_path = 'stringIO object'
#        gene_name = 'unknown_gene'

    all_genotypes = []
    individuals = []
    filteredout_genotypes = set([])
    if individuals_dict is None or individuals_dict is False:
        individuals_dict = {}
    for line in binary_file:
        if not line.strip() == '':
            if not line.startswith('#'):
#                print line
                if not re.match('^\d+$', line):
#               if line.startswith('neutral_sim'):
                    fields = line.split()
                    gene_name = fields[0]
                    individual = fields[1]
                    individual_key = fields[1].replace("_1", "").replace("_2", "").replace("_a", "").replace("_b", "")
                    individuals_dict.setdefault(individual_key, {})
#                    population = fields[2]
                    individuals_dict[individual_key]['subpop'] = fields[2]
#                    continent = fields[3]
                    individuals_dict[individual_key]['pop'] = fields[3]
                    individuals_dict[individual_key]['continent'] = fields[3]
                    individuals_dict[individual_key].setdefault('techniques', '')
#                    sex = fields[4]
                    individuals_dict[individual_key]['sex'] = 'X'
                    genotypes = fields[5:]

#                    print "AAAAAAAA", fields[0:9]
#                    if downstream is not None:
#                        downstream = min(len(genotypes), downstream)
#                    else:
#                        downstream = len(genotypes)
#
                    # TODO: this can be done better using numpy's tools
                    for pos in xrange(len(genotypes)):
                        if genotypes[pos] not in ("0", "1"):
                            filteredout_genotypes.update([pos])

#                    all_genotypes.append(''.join(genotypes[upstream:downstream]))
                    all_genotypes.append(''.join(genotypes))
                    individuals.append(individual)
    if len(all_genotypes) == 0:
        raise GenotypeNetworkInitError("No genotypes in {0}".format(binaryfile_path))
#        continue
    total_number_snps = len(all_genotypes[0])

    # Filter out all SNPs that have at least one genotype different from 0 or 1
    if len(filteredout_genotypes) > 0:
        kept = [n for n in range(total_number_snps) if n not in filteredout_genotypes]
        logging.warn("Triallelic SNP found in {0}".format(binaryfile_path))
        if len(kept) == 0:
            error_message = "No valid genotypes found for {0} after filtering".format(binaryfile_path)
            logging.warn(error_message)
            raise GenotypeNetworkInitError(error_message)
#            continue
        get_notnull_genotypes = operator.itemgetter(*kept)
        logging.debug("removed {0} genotypes out of {1} after filtering for MAF and for wrong genotypes".format(len(filteredout_genotypes), len(all_genotypes[0])))
#        print kept
        all_genotypes = [''.join(get_notnull_genotypes(genotypes)) for genotypes in all_genotypes]
#        print all_genotypes
#    print(filteredout_genotypes)
#    print(len(all_genotypes), all_genotypes[:10])
    return all_genotypes, individuals, individuals_dict, gene_name

#def generate_network(binary_file, individuals_dict=None, distance=1, 
#        upstream=None, downstream=None, n_individuals = 0, random_seed=False,
#        window_size=None, overlapping_windows=None):


def generate_network(all_genotypes, individuals, individuals_dict, gene_name,
        distance = 1, upstream=None, downstream = None, n_individuals = 0, 
        random_seed = False, window_size = None, overlapping_windows = None,
        use_hdf5 = False):
    """
    Given a set of genotypes, generate a genotype network.


    # TODO: update documentation, after splitting the function and adding use_hdf5 option

    ++++++++++
    Parameters
    ++++++++++
    
        binary_file
            binary file object or path (see below for example of format)

        individuals_dict
            file containing individuals and population definitions

        distance
            definition of distance between neighbors in the genotype space
        
        upstream, downstream
            positions to extract only a subset of SNPs from the data. Start position is included.

    +++++++++++++++++++
    Example and doctest
    +++++++++++++++++++

    >>> from StringIO import StringIO
    >>> binary_file = StringIO()
    >>> sample_binary_file = r'''#
    ... # gene  chromosome_id rs1 rs2 rs3 rs4 rs5 rs6 rs7
    ... MOGS HG00096_1 GBR EUR X 0 0 0 1 1 0 0
    ... MOGS HG00096_2 GBR EUR X 0 0 0 1 1 0 0
    ... MOGS HG00097_1 GBR EUR X 0 0 0 0 0 0 0
    ... MOGS HG00097_2 GBR EUR X 0 0 0 0 0 0 0
    ... MOGS HG00099_1 GBR EUR X 0 0 0 0 0 0 1
    ... MOGS HG00099_2 GBR EUR X 0 0 0 0 0 0 1
    ... MOGS HG00100_1 GBR EUR X 0 0 0 0 1 0 0
    ... MOGS HG00100_2 GBR EUR X 0 0 0 0 1 0 0
    ... MOGS HG00101_1 GBR EUR X 1 0 0 0 1 0 0
    ... MOGS HG00101_2 GBR EUR X 1 0 0 0 1 0 0
    ... '''
    >>> binary_file.write(sample_binary_file)
    >>> binary_file.seek(0)
    >>> individuals_dict = parse_individuals_file()
    >>> genotypes, individuals, individuals_dict, gene_name = parse_binary_file(binary_file, individuals_dict)
    >>> network = generate_network(genotypes, individuals, individuals_dict, "MOGS")
    >>> print network.summary() #doctest: +NORMALIZE_WHITESPACE
    5 nodes, 4 edges, undirected
    <BLANKLINE>
    Number of components: 1
    Diameter: 3
    Density: 0.4000
    Average path length: 1.8000
    >>> print network['individuals']
    ['HG00096_1', 'HG00096_2', 'HG00097_1', 'HG00097_2', 'HG00099_1', 'HG00099_2', 'HG00100_1', 'HG00100_2', 'HG00101_1', 'HG00101_2']

    
    """

        
#    print "GENO:", all_genotypes[0:5]
#    print set([len(geno) for geno in all_genotypes])
    if upstream is None:
        upstream = 0
    else:
        upstream = upstream - 1
    if downstream is None:
        downstream = len(all_genotypes[0])
    else:
        downstream = min(downstream, len(all_genotypes[0]))
    if upstream > downstream:
        raise ValueError("Window Start can not be bigger than Window End")

    # SUBSAMPLE NETWORK, if required
    if random_seed is not False:
        random.seed(random_seed)
    if n_individuals > 0:
        if n_individuals < len(all_genotypes):
            all_genotypes = random.sample(all_genotypes, n_individuals)

    chromosome_len = downstream-upstream

    # GENERATE NETWORK
    network = GenotypeNetwork(chromosome_len, name=gene_name)
    if use_hdf5 is True:
        all_genotypes = all_genotypes[:, upstream:downstream]
        all_genotypes = ["".join([str(geno) for geno in indv_genos]) for indv_genos in all_genotypes]
    else:
        all_genotypes = [genotype[upstream:downstream] for genotype in all_genotypes]
#    print gene_name, all_genotypes[0:5]

#    print all_genotypes[0:5]
    network.populate_from_binary_strings_and_individuals(
            genotypes=all_genotypes, distance=distance, 
            individuals=individuals, individuals_dict=individuals_dict)
    return network

def generate_network_wrapper(options):
    """wrapper function to run generate_network in parallel

    """
    binary_file_path = options[0]
    individuals_dict = options[1]
    return generate_network(binary_file_path, individuals_dict)



def print_report_and_subgraphs(allpops_binaryfile, gene, individuals_dict, 
            report_type, report_fields, options, 
            central_window_only=False, random_seed=0, 
            phenotype="continent", 
            pops=False,         # DEPRECATED
            n_individuals=0    # DEPRECATED
            ): 
    """
    Given the path to a binary file, generate a report of network properties 
    of the global graph and of the subpopulations.

    Parameters:
    - allpops_binary_file   -> handler to a .binary file (see filteredvcf_to_binary.py)
    - gene                  -> a label for the gene, region, or window. This will be the name of the network.
    - individuals_dict      -> dictionary containing list of individuals and populations. Recommended to leave it False.
    - pops                  -> populations for which to calculate the subnetwork. 
                                If empty, only the properties of the global network will be calculated. 
                                Recommended: ['EUR', 'ASN', 'AFR'].
    - report_type           -> whether to print a report per every node or per every network.
    - report_fields         -> which fields to include in the report.
    - options               -> misc options, including windows_size, distance_definition, upstream and downstream position...
    - central_window_only   -> If True, only the properties of the central window will be calculated.
    - n_individuals         -> If >0, sample only a subset of individuals for every population.
    - random_seed           -> seed for subsampling individuals randomly.
    - phenotype             -> the phenotype by which splitting individuals in different networks (default: by continent)
    """
#    print allpops_binaryfile.name
    skip_gene = False
    networks = {}
    networks[gene] = {}
    networks[gene]['glob'] = {}

    snp_ids = []
    chromosomes = []
    positions = []
    ref_alleles = []
    n_snps = None
#    report_fields.insert(1, phenotype)
#    print report_fields

    if options.use_hdf5 is False:
        while n_snps is None:
            current_line = allpops_binaryfile.readline()
            if current_line.startswith('#'):
                if current_line.startswith('#SNP IDS:'):
                    snp_ids = current_line.split()[2:]
                if current_line.startswith('#Chromosome:'):
                    chromosomes = current_line.split()[1:]
                if current_line.startswith('#Position:'):
                    positions = [int(p) for p in current_line.split()[1:]]
                if current_line.startswith('#Ref. allele:'):
                    ref_alleles = current_line.split()[2:]
            else:
                snps_str = re.findall("^.*?\s+?([01 ]*)$", current_line)
                if snps_str != []:
                    n_snps = len(snps_str[0].split())
            if current_line == '': # if current_line is empty, it means that we reached the downstream of the file
                logging.warn("The Binary File for {0} is not valid".format(allpops_binaryfile.name))
                skip_gene = True
                break
        allpops_binaryfile.seek(0)
    elif options.use_hdf5 is True: # TODO: what happens if the HDF5 file doesn't contain snp_annotations?
        import h5py
#        print allpops_binaryfile.name
        try:
            hdf5_file = h5py.File(allpops_binaryfile.name, 'r')
        except:
            logging.warn("The HDF5 file for {0} is not found or could not be opened".format(allpops_binaryfile.name))
            return
        snp_annotations = hdf5_file['snp_annotations']
        snp_ids = snp_annotations[:, 0]
        chromosomes = snp_annotations[:, 1]
        positions = [int(p) for p in snp_annotations[:, 2]]
#        print positions[0:5]
        ref_alleles = snp_annotations[:, 3]
        n_snps = len(snp_ids)

#    print (snp_ids, chromosomes, positions, ref_alleles)
    if skip_gene is True:
        return
    
    # if step is not a multiple of n_snps, remove some snps from the beginning and the ned
    if central_window_only is True:
        if options.windows_size == 0:
            windows_size = n_snps   
            step = n_snps
            skipped_initial_snps = 0
        else:
            windows_size = options.windows_size
            step = windows_size
            skipped_initial_snps = (n_snps / 2) - (windows_size / 2)
        n_snps = skipped_initial_snps + windows_size
#        print (skipped_initial_snps, n_snps, step)
#        print range(skipped_initial_snps, n_snps, step)
    else:
        if options.windows_size == 0:
            windows_size = n_snps 
            step = n_snps
            skipped_initial_snps = 0
        else:
            windows_size = options.windows_size
            if options.overlapping is not True:
                # non-overlapping windows
                step = windows_size
            else:
                step = 1
            skipped_initial_snps = int((n_snps % step) / 2)

    if options.use_hdf5 is False:
        try:
            all_genotypes, individuals, individuals_dict, gene_name = parse_binary_file(allpops_binaryfile, individuals_dict)
        except:
            return None
    elif options.use_hdf5 is True:
        try:
            all_genotypes, individuals, individuals_dict, gene_name = parse_hdf5_file(allpops_binaryfile, individuals_dict)
        except:
            return None

#    print "genotypes parsed"
#    print all_genotypes[0:5]
    original_random_seed = random_seed
    for window_start in range(skipped_initial_snps, n_snps, step):
        window_end = window_start+windows_size
        random_seed = original_random_seed
#        print skipped_initial_snps, step, window_start, window_end
#        print ">", gene, window_start, window_end, windows_size, n_snps
#        print window_end < n_snps
        if window_end > n_snps:
            continue

        try:

            # GENERATE NETWORK AND CALCULATE PROPERTIES
            allpops_network = generate_network(
                                            all_genotypes = all_genotypes,
                                            individuals = individuals,
                                            individuals_dict = individuals_dict,
                                            gene_name = gene_name,
#                                            individuals_dict=individuals_dict, 
                                            distance = options.distance, 
                                            upstream = window_start+1, #NOTE: window_start is corrected in generate_network. For this reason, I am adding +1 here
                                            downstream = window_end,
                                            use_hdf5 = options.use_hdf5)
            allpops_network['filename'] = allpops_binaryfile.name
            allpops_network['file_prefix'] = gene_name
            allpops_network[phenotype] = 'glob'
        except GenotypeNetworkInitError:
            # skip the current window
            continue

        window_label = 'window_' + str(window_start / step) 
#        networks[gene]['glob'][window_label] = allpops_network
        allpops_network['window'] = window_label
        allpops_network['whole_gene_nsnps'] = n_snps
        allpops_network['gene'] = gene
        if snp_ids != []:
            allpops_network['snp_ids'] = snp_ids[window_start:window_end]
        else:
            allpops_network['snp_ids'] = ['NA'] * windows_size
        if chromosomes != []:
            allpops_network['chromosomes'] = chromosomes[window_start:window_end]
        else:
            allpops_network['chromosomes'] = ['NA'] * windows_size
        if positions != []:
            allpops_network['positions'] = positions[window_start:window_end]
            allpops_network['distance_from_upstream_margin'] = min(positions[window_start:window_end]) - min(positions)
            allpops_network['distance_from_downstream_margin'] = max(positions) - max(positions[window_start:window_end])
        else:
            allpops_network['positions'] = [numpy.nan] * windows_size
            allpops_network['distance_from_upstream_margin'] = numpy.nan
            allpops_network['distance_from_downstream_margin'] = numpy.nan
        if ref_alleles != []:
            allpops_network['ref_allele'] = ref_alleles[window_start:window_end]
        else:
            allpops_network['ref_allele'] = ['N'] * windows_size


        # PRINT REPORT TO STDOUT
        if report_type == 'values_by_window':
#            report_output += allpops_network.csv_report(report_fields)
#            print allpops_network
            print allpops_network.csv_report(report_fields)
        if report_type == 'values_by_node':
#            report_output += allpops_network.values_by_node(report_fields, header=False)
            print allpops_network.values_by_node(report_fields, header=False)
#        if report_type == "individuals":
#            pass
#            print "-----"
#            print gene_name, "allpops"
#            for ind in allpops_network['individuals']:
#                ind_key = ind.replace('_1', '').replace('_2', '')
#                print ind, individuals_dict[ind_key]['continent'],
#                print individuals_dict[ind_key]['subpop']

        if options.use_hdf5 is False:
            allpops_binaryfile.seek(0)




# SUBSAMPLING BY POPULATION OR PHENOTYPE
        if phenotype == 'continent':
            pops = allpops_network['allcontinents']
        else:
            if allpops_network['allphenotype_statuses'].has_key(phenotype):
                pops = allpops_network['allphenotype_statuses'][phenotype]
            else:
                raise InvalidPhenotype("""

    The phenotype selected is not defined in the input file you provided. Check the individuals file

    This is the phenotype you requested: {0}. There is no column for this phenotype in {1}.

""".format(phenotype, options.individuals_file), options.individuals_file, phenotype)

        if options.n_individuals > 0:
            pops.insert(0, 'global_sub')
#        print pops
        output = ""
#        report_fields.replace("continent", phenotype)
        for pop in pops:
#            print pop

            random_seed = sum([ord(l) for l in pop]) + random_seed # make sure that a different (yet reproducible) seed is used for every resampling
#            print random_seed
            subnetwork = allpops_network.subgraph_by_phenotype(phenotype, pop, n_individuals=options.n_individuals, random_seed=random_seed)
            if subnetwork is None:
                continue
            subnetwork[phenotype] = pop
#            print subnetwork.attributes
            if subnetwork is not None and subnetwork.vcount() > 0:
                if report_type == 'values_by_window':
                    print subnetwork.csv_report(report_fields)
                if report_type == 'values_by_node':
                    print subnetwork.values_by_node(report_fields, header=False)
                if report_type == "individuals_line":
                    output += "-----"
                    output += gene_name + ' ' + pop + ' '
                    for ind in subnetwork['individuals']:
                        ind_key = ind.replace('_1', '').replace('_2', '')
#                        output += ' ' + ind + ' ' + individuals_dict[ind_key]['continent'] + ' '
                        output += ' ' + ind + ' ' + individuals_dict[ind_key]['continent'] + ' '
                        output += individuals_dict[ind_key]['subpop'] + ' '

                if report_type == "individuals":
                    print "-----"
                    print gene_name, pop
                    for ind in subnetwork['individuals']:
                        ind_key = ind.replace('_1', '').replace('_2', '')
#                        print ind, individuals_dict[ind_key]['continent'], 
                        print ind, individuals_dict[ind_key]['continent'], 
                        print individuals_dict[ind_key]['subpop']

        if report_type == "individuals_line":
            print output




if __name__ == '__main__':
    (binary_file_path, options) = get_options()
    individuals_dict = parse_individuals_file(options.individuals_file)
    logging.debug("starting parsing {0}".format(binary_file_path))
    if options.use_hdf5 is False:
        all_genotypes, individuals, individuals_individuals_dict, gene_name = parse_binary_file(binary_file_path, individuals_dict)
    elif options.use_hdf5 is True:
        raise NotImplemented
        all_genotypes, individuals, individuals_individuals_dict, gene_name = parse_hdf5_file(open(binary_file_path, 'r'), individuals_dict)
    network = generate_network(all_genotypes, individuals,
            individuals_dict, gene_name=gene_name,
            distance=options.distance, upstream=options.upstream, 
            downstream=options.downstream)
    logging.debug("network parsed correctly. Starting writing csv report")
    print network.csv_report()





#!/usr/bin/env python
"""
Test the get_genotypes script, to download vcf files from 1000genomes
"""
import subprocess
import os
import logging
import nose

@nose.plugins.attrib.attr('internet')
def test_SimpleSetOfGenes():
    coordsfile = 'test/test_data/coords/n-glycan.coords'
    expected_outputfiles = "test/test_data/vcf/ALG11.vcf.gz test/test_data/vcf/NRG3.vcf.gz".split()
    logging.debug( "rm {0}".format(" ".join(expected_outputfiles)))
    output = subprocess.check_output("rm -f {0}".format(" ".join(expected_outputfiles)).split())

    com = "python src/get_genotypes.py --coordinates_file {0} --outputpath test/test_data/vcf/ --flank 10000 --force-download --report_output test/test_data/vcf/n_snps_report.txt".format(coordsfile)
    output = subprocess.check_output(com.split())
    for f in expected_outputfiles:
        yield check_file_exists, f
    

def check_file_exists(f):
    assert os.stat(f).st_size > 0


if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
"""
Check that creating a network with or without specifying individuals leads to the same network
"""
import unittest
import nose
from GenotypeNetwork import GenotypeNetwork

class SimpleNetwork(unittest.TestCase):
    name = "simple_network"
    chromosome_len = 3
#                   EUR    EUR    AFR    AFR    AFR
    genotypes =   ['001', '000', '000', '000', '010']
    individuals = ['HG1', 'HG2', 'HG3', 'HG4', 'HG5']
    individuals_dict = {
                    'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                    'HG2': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'bad'}, 
                    'HG3': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                    'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                    'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
    }

    def setUp(self): # TODO: see if you can use setUpAll
        self.network_with_individuals = GenotypeNetwork(self.chromosome_len, name = self.name)
        self.network_with_individuals.populate_from_binary_strings_and_individuals(genotypes=self.genotypes, individuals=self.individuals, 
                                individuals_dict=self.individuals_dict)

        self.network_without_individuals = GenotypeNetwork(self.chromosome_len, name = self.name)
        self.network_without_individuals.populate_from_binary_strings(genotypes=self.genotypes)

    def test_individuals(self):
#        nose.tools.assert_sequence_equal(self.network_with_individuals['individuals'], self.individuals)
        nose.tools.assert_equal(self.network_with_individuals['individuals'], self.individuals)

    def test_summary(self):
#        nose.tools.assert_sequence_equal(self.network_with_individuals.summary(), self.network_without_individuals.summary())
        nose.tools.assert_equals(self.network_with_individuals.summary(), self.network_without_individuals.summary())

    def test_csv_report(self):
#        nose.tools.assert_sequence_equal(self.network_with_individuals.csv_report(), self.network_without_individuals.csv_report())
        nose.tools.assert_equals(self.network_with_individuals.csv_report(), self.network_without_individuals.csv_report())

class DiploidNetwork(SimpleNetwork):
#    self.network = GenotypeNetwork(3, 'sample_phenotype')
    name = "diploid_network"
    chromosome_len = 3
#                   EUR       EUR      EUR      EUR      AFR      AFR      AFR      AFR      AFR      AFR
    genotypes =   ['001',    '000',   '000',   '000',   '010',   '100',   '101',   '111',   '000',   '101']
    individuals = ['HG1_1', 'HG1_2', 'HG2_1', 'HG2_2', 'HG3_1', 'HG3_2', 'HG4_1', 'HG4_2', 'HG5_1', 'HG5_2']
    individuals_dict = {
                    'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                    'HG2': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'bad'}, 
                    'HG3': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
                    'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                    'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
    }


if __name__ == '__main__':
    import logging
    import doctest
    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()





#!/usr/bin/env python
"""
----------------------------------
Genotype Network class and functions
----------------------------------

"""
#import igraph0 as igraph
import igraph
import itertools
import logging
#import pprint
import operator
#import cPickle
import random
import numpy
import igraph.vendor.texttable
try:
    import collections
except:
    from src import collections as collections
#import re
import ast
from custom_exceptions import *


class GenotypeNetwork(igraph.Graph):
    """
    Represents a Genotype Network

    ==============
    Description
    ==============
    
    Class to represent a GenotypeNetwork in python. 
      Uses the igraph C library and its python bindings.

    See the file docs/GenotypeNetworkTutorial.py for more examples and 
      doctests on how to use this class.

    ==============
    Parameters:
    ==============

    * chromosome_len    -> size of chromosomes
    * name              -> descriptive name for the network
    * parallelize       -> if True, use multicore module to build the network (not implemented yet)
    * args              -> other arguments to be passed to igraph.Graph

    ==============
    Examples
    ==============

    Create an empty Genotype Network:
    >>> phenotype1 = GenotypeNetwork(chromosome_len=3, name='phenotype1')

    Populate the graph from a set of genotypes:
    >>> genotypes = ['001', '010', '000']
    >>> phenotype1.populate_from_binary_strings(genotypes)
    
    # Print informations about the network:
    >>> print(phenotype1)
    Genotype Network (name = 'phenotype1', chromosome lenght = 3, |V| = 3, |E| = 2)

    >>> print(phenotype1.summary()) #doctest: +NORMALIZE_WHITESPACE
    3 nodes, 2 edges, undirected
    <BLANKLINE>
    Number of components: 1
    Diameter: 2
    Density: 0.6667
    Average path length: 1.3333

    # Print node degree distribution for the network
    >>> phenotype1.degree()
    [1, 1, 2]


    """
    def __init__(self, chromosome_len, name='', genotypes=None, parallelize=False, *args, **kwd):
        igraph.Graph.__init__(self, name=name, *args, **kwd)
        self['name'] = name
        self['chromosome_len'] = int(chromosome_len)
        self["pop"] = "glob"
        self["continent"] = "glob"
        self["snp_ids"] = ["NA"] * chromosome_len
        self["chromosomes"] = ["NA"] * chromosome_len
        self["positions"] = [numpy.nan] * chromosome_len
        self["ref_alleles"] = ["NA"] * chromosome_len

        if genotypes is not None:
            self.populate_from_binary_strings(genotypes, 
                            parallelize=parallelize)
 
        self.layout_initialized = False
        self.network_properties_calculated = False

    def __str__(self):
        """
        determine how the GenotypeNetwork will be printed

        >>> phenotype1 = GenotypeNetwork(5, 'myphenotype')
        >>> print phenotype1
        Genotype Network (name = 'myphenotype', chromosome lenght = 5, |V| = 0, |E| = 0)

        """
        representation = "Genotype Network (name = '{0}', chromosome lenght = {1}, |V| = {2}, |E| = {3})".format(
                                                self['name'], self['chromosome_len'], self.vcount(), self.ecount())
        return representation

    def summary(self):
        """
        Mimics the old igraph 0.5.4 summary() output.
    

        Example summary:

        >>> phenotype1 = GenotypeNetwork(chromosome_len=3, name='phenotype1')

        Populate the graph from a set of genotypes:
        >>> genotypes = ['001', '010', '000']
        >>> phenotype1.populate_from_binary_strings(genotypes)
        >>> print (igraph.Graph.summary(phenotype1))
        IGRAPH U--- 3 2 -- phenotype1
        + attr: chromosome_len (g), chromosomes (g), continent (g), distance_definition (g), individuals (g), name (g), pop (g), positions (g), ref_alleles (g), snp_ids (g), genotype (v), n_datapoints (v), value (v)

        >>> print(phenotype1.summary())
        3 nodes, 2 edges, undirected
        <BLANKLINE>
        Number of components: 1
        Diameter: 2
        Density: 0.6667
        Average path length: 1.3333

        """
        template = "{0} nodes, {1} edges, undirected\n\nNumber of components: {2}\nDiameter: {3}\nDensity: {4:.4f}\nAverage path length: {5:.4f}"
        output = template.format(self.vcount(), self.ecount(), len(self.components()), self.diameter(), self.density(), self.average_path_length())
        return(output)



    def __get_layout__(self):
        """
        Calculate the layout of the graph for plotting purposes.

        >>> phenotype1 = GenotypeNetwork(3, genotypes=['011', '010', '000'])
        >>> phenotype1.__get_layout__()
        >>> print phenotype1.vs['pos_x']
        [1, 2, 0]
        >>> print phenotype1.vs['pos_y']
        [0.5, -0.5, 0.5]
        >>> print phenotype1.vs['layout']
        [(1, 0.5), (2, -0.5), (0, 0.5)]

        This is an internal function called by self.plot() when needed.

        """
        if self['chromosome_len'] > 10:
            raise NotImplementedError(
                    "sorry, the network is too large to calculate the layout. Future version will be more efficient and allow to generate the layout of larger networks")
        self.vs['pos_x'] = [n['genotype'].count('1') for n in self.vs]
        self.vs['pos_y'] = [get_y_position(n['genotype']) for n in self.vs]
        self.vs['layout'] = zip(self.vs['pos_x'], self.vs['pos_y'])
        self.layout_initialized = True

    def __calculate_network_properties__(self):
        """
        Internal function to calculate many interesting network properties. 
        
        These will be stored as properties of the GenotypeNetwork object, and available when it is exported to graphML, and to functions such as self.csv_report()

        This function is not called unless one of the functions that need it is called (e.g. write_graphml, csv_report). 
        >>> mynet = GenotypeNetwork(chromosome_len=4,  name="Mygene", genotypes=["0000", "0000","0001", "0010", "0111"])
        >>> print mynet.csv_report()
        Mygene glob 4 5 4 2 2 2 0.3333 1.25 1.3333 1.0 2 0.25 0.4018

        n_genotypes is the number of genotypes used to generate the network. 
        In this case the number of nodes is 4, but n_genotypes is 5, because two genotypes ("0000" and "0000") were the same.
        >>> print mynet['n_genotypes']
        5

        The density is calculated by the igraph library
        >>> print mynet['density']
        0.3333

        av_datapoints_per_node is not related to density. Instead, it represents how many genotypes, on average, represent every node.
        In this case, it is between 1 and 2, as three nodes have 1 genotype, and one node ("0000") has 2.
        >>> print mynet['av_datapoints_per_node']
        1.25

        For other properties, see the documentation of igraph.
        >>> print mynet['n_components']
        2

        >>> print mynet['av_degree']
        1.0
        """
        # DESCRIPTIVE PROPERTIES
        # if gene attribute is not defined, define it as the network name
        if not 'gene' in self.attributes():
            self['gene'] = self['name']
        self['n_snps'] = self['chromosome_len']
        self['n_genotypes'] = sum(self.vs['n_datapoints'])
#        self['n_genotypes'] = sum([len(individuals) for individuals in self.vs['node_individuals']])
        self['n_datapoints'] = self['n_genotypes']
        self['av_datapoints_per_node'] = round(numpy.mean(self.vs['n_datapoints']), 4)
        self['av_datapoints_per_vertex'] = self['av_datapoints_per_node']
        self.vs['perc_datapoints'] = numpy.divide(self.vs['n_datapoints'], 1.0 * sum(self.vs['n_datapoints']))
#        print("source and target:")
        self.es['perc_datapoints'] = [sum((self.vs(e.source)['perc_datapoints'][0], self.vs(e.target)['perc_datapoints'][0])) for e in self.es()]
#        print self.vs['n_datapoints']
#        print self.es['perc_datapoints']
        self.es['inv_perc_datapoints'] = numpy.divide(1.0, self.es['perc_datapoints'])
        self.vs['inv_perc_datapoints'] = numpy.divide(1.0, self.vs['perc_datapoints'])
#        print self.es['perc_datapoints']

        self['n_vertices'] = self.vcount()
        self['n_nodes'] = self['n_vertices']
        self['n_edges'] = self.ecount()
        self['n_components'] = len(self.components())


        # SNPs AND PHYSICAL POSITION PROPERTIES
#        print self['n_snps'], self['snp_ids'], len(self['snp_ids'])
        central_snp_index = (self['n_snps']-1)/2
        self['central_snp_position'] = self['positions'][central_snp_index]
        self['central_snp'] = self['snp_ids'][central_snp_index] # if the total number of SNPs is 10, the central SNP is the one in position 4 (or 5 counting from 1)
        self['chromosome'] = list(set(self['chromosomes']))[0]
        self['downstream_position'] = min(self['positions'])
        self['upstream_position'] = max(self['positions'])
#        print self.vs['positions']
#        print self['positions']
        if operator.isNumberType(self['upstream_position']) and not self['upstream_position'] is numpy.nan:
            self['region_size'] = int(self['upstream_position']) - int(self['downstream_position'])
        else:
            self['region_size'] = numpy.nan


        # PATH LENGTH PROPERTIES
        self['av_path_length'] = round(self.average_path_length(), 4)
#        print "edge weights (based on percentage of data points in the nodes connected):"
#        print self.es['perc_datapoints']
#        print [e.tuple for e in self.es()]
#        print self.shortest_paths(weights=self.es['perc_datapoints'])
        if self.ecount() == 0:
            self['av_w_path_length'] = numpy.nan
            self['av_w_path_length_inv'] = numpy.nan
        else:
            self['av_w_path_length'] = round(numpy.mean([path_len for path_len in sum(self.shortest_paths(weights=self.es['perc_datapoints']), []) if path_len > 0 and path_len != float('inf')]), 4)
            self['av_w_path_length_inv'] = round(numpy.mean([path_len for path_len in sum(self.shortest_paths(weights=self.es['inv_perc_datapoints']), []) if path_len > 0 and path_len != float('inf')]), 4)
#            self['av_w_path_length'] = round(numpy.mean([path_len for path_len in sum(self.shortest_paths(), []) if path_len > 0 and path_len != float('inf')]), 4)
#            self['av_w_path_length_inv'] = round(numpy.mean([path_len for path_len in sum(self.shortest_paths(), []) if path_len > 0 and path_len != float('inf')]), 4)
        self['var_path_length'] = round(self.path_length_hist().var, 4)
#        self['min_path_length'] = round(self.average_path_length(), 4)
#        self['max_path_length'] = round(self.average_path_length(), 4)
        self['diameter'] = self.diameter()


        ## DEGREE PROPERTIES
        self['av_degree'] = round(numpy.mean(self.degree()), 4)
        self['av_w_degree'] = round(numpy.mean(numpy.multiply(self.degree(), self.vs['perc_datapoints'])), 4)
        self['av_w_degree_inv'] = round(numpy.mean(numpy.multiply(self.degree(), self.vs['inv_perc_datapoints'])), 4)
        self['var_degree'] = round(numpy.var(self.degree()), 4)
        self['median_degree'] = round(numpy.median(self.degree()), 4)
        self['max_degree'] = self.maxdegree()

        self['av_closeness'] = round(numpy.mean(self.closeness()), 4)
        self['var_closeness'] = round(numpy.var(self.closeness()), 4)
        self['median_closeness'] = round(numpy.median(self.closeness()), 4)
        self['max_closeness'] = get_max(self.closeness())

        self['density'] = round(self.density(), 4)

        self['av_betweenness'] = round(numpy.mean(self.betweenness()), 4)
        self['var_betweenness'] = round(numpy.var(self.betweenness()), 4)
        self['median_betweenness'] = round(numpy.median(self.betweenness()), 4)
        self['max_betweenness'] = get_max(self.betweenness())
      
        self.network_properties_calculated = True


    def write_graphml(self, *args, **kwd):
        """
        The igraph 0.6 version doesn't support storing dict attributes to graphml files.

        This function is a wrapper to convert all dict attributes to strings before calling write_graphml.
        """
        newnetwork = self
        for attribute in newnetwork.attributes():
#            print "ATTRIBUTE: ", attribute, type(newnetwork[attribute]), type(newnetwork[attribute]) == dict
            if type(newnetwork[attribute]) == dict:
#                print "STORING", attribute, newnetwork 
                newnetwork[attribute] = str(self[attribute])
            if type(newnetwork[attribute]) == list:
#                print "STORING", attribute, newnetwork 
                newnetwork[attribute] = str(self[attribute])
#                print newnetwork[attribute][:30], type(newnetwork[attribute])
#        print [(a, type(a)) for a in newnetwork.attributes()]
        igraph.Graph.write_graphml(newnetwork, *args, **kwd)

    def populate_from_binary_strings(self, genotypes, 
                                            distance=1,
                                            parallelize=False):
        """
        populate the GenotypeNetwork from a list of genotypes
        >>> phenotype1 = GenotypeNetwork(3)
    
        ==========
        Parameters
        ==========

        genotypes must be a list containing strings of the same length, and composed only by "0" and "1" characters. 

        >>> genotypes = ['001', '010', '000', '000', '100', '000'] # note: duplicated genotypes are merged
        >>> phenotype1.populate_from_binary_strings(genotypes)
        >>> print phenotype1
        Genotype Network (name = '', chromosome lenght = 3, |V| = 4, |E| = 3)

        >>> print phenotype1.vs['genotype']
        ['010', '001', '000', '100']

        >>> print phenotype1.vs['value']
        [2, 1, 0, 4]

        >>> print phenotype1.summary() #doctest: +NORMALIZE_WHITESPACE
        4 nodes, 3 edges, undirected
        <BLANKLINE>
        Number of components: 1
        Diameter: 2
        Density: 0.5000
        Average path length: 1.5000

        See the file docs/GenotypeNetworkTutorial.py for more examples and doctests on how to use this class.

        """
        if parallelize is True:
            raise NotImplementedError
        if not (type(genotypes) in (list, tuple) ) or not (set([len(x) for x in genotypes]) == set([self['chromosome_len']])):
            raise GenotypeNetworkInitError("The list of genotypes provided to .populate_from_binary_strings is incorrect. {0} (network {1})".format(str(genotypes), self['name']))
        
        self['distance_definition'] = distance
        self['individuals'] = None

        genotype_counts = collections.Counter(genotypes)
        node_genotypes = []
        node_values = []
        node_n_datapoints = []
        self.add_vertices(len(genotype_counts.keys()))
        for (genotype, geno_count)  in genotype_counts.items():
            node_genotypes.append(genotype)
            try:
                node_values.append(int(genotype, base=2))
            except ValueError:
                raise GenotypeNetworkInitError("something went wrong when converting genotypes to binary strings. Check if any SNP is triallelic")
            node_n_datapoints.append(geno_count)
        self.vs['genotype'] = node_genotypes
        self.vs['value'] = node_values
        self.vs['n_datapoints'] = node_n_datapoints
        for genotype1 in self.vs:
#            logging.debug(str((genotype1.index, genotype1['name'], genotype1['value'])))
            for genotype2 in self.vs:
                if genotype1.index >= genotype2.index:
                    pair_distance = '{0:08b}'.format(genotype1['value']^genotype2['value']).count('1')
                    if pair_distance > 0 and pair_distance <= distance:
#                    print genotype1['name'], genotype2['name'],
#                    print '{:08b}'.format(genotype1['value']^genotype2['value']).count('1')
                        self.add_edge(genotype1.index, genotype2.index)


    def populate_from_binary_strings_and_individuals(self, genotypes,
                                                            distance=1,
                                                            individuals=False,
                                                            individuals_dict=False,
                                                            parallelize=False):
        """
        ======================
        Adding individual info
        ======================

        In the newer versions, it is possible to add information on individuals 
            and populations to the network.

        To do so, you need to provide a list of individuals, and a dictionary 
            containing the annotations on the individuals.

        The list of individuals must be of the same length as the genotypes, 
            and it will be assumed that there is correspondence between positions, 
            so that the genotype of the individual in the first position is also 
            in the first position of the genotypes list.

        >>> genotypes = ['001', '000', '000', '000']
        >>> individuals = ['HG1', 'HG2', 'HG3', 'HG4']
        >>> individuals_dict = {
        ...             'HG1': {'continent': 'EUR', 'subpop': 'TSC', 
        ...                     'phenotype1': 'ILLUMINA', 'phenotype2': 'tall', 'phenotype3': 'good'}, 
        ...             'HG2': {'continent': 'EUR', 'subpop': 'GBR', 
        ...                     'phenotype1': 'ABI_SOLID', 'phenotype2': 'tall', 'phenotype3': 'bad'}, 
        ...             'HG3': {'continent': 'AFR', 'subpop': 'AFR', 
        ...                     'phenotype1': 'ILLUMINA', 'phenotype2': 'short', 'phenotype3': 'neutral'}, 
        ...             'HG4': {'continent': 'AFR', 'subpop': 'YRI', 
        ...                     'phenotype1': 'ILLUMINA', 'phenotype2': 'tall', 'phenotype3': 'bad'}, 
        ... }
        >>> mynet = GenotypeNetwork(3)
        >>> mynet.populate_from_binary_strings_and_individuals(genotypes, individuals=individuals, individuals_dict=individuals_dict)
        >>> mynet.vs['genotype']
        ['001', '000']
        >>> mynet.vs['value']
        [1, 0]
        >>> mynet.vs['n_datapoints']
        [1, 3]
        >>> mynet.vs['node_individuals'] 
        [['HG1'], ['HG4', 'HG2', 'HG3']]
        >>> mynet.vs['continent']
        [['EUR'], ['AFR', 'EUR']]
        >>> mynet.vs['subpop']
        [['TSC'], ['AFR', 'GBR', 'YRI']]
        >>> mynet.vs['phenotype1']
        [['ILLUMINA'], ['ABI_SOLID', 'ILLUMINA']]
        >>> mynet.vs['phenotype2'] # the first node is only "tall", the second is both "tall" and "short"
        [['tall'], ['short', 'tall']]
        >>> mynet.vs['phenotype3'] # the first node is only "tall", the second is both "tall" and "short"
        [['good'], ['bad', 'neutral']]

        >>> print mynet.vs[1]['phenotype2']  # phenotype2 of node 000
        ['short', 'tall']

        >>> print mynet.vs[1]['phenotype1']  # phenotype2 of node 000
        ['ABI_SOLID', 'ILLUMINA']

        BEWARE: the order of individuals is shuffled after adding them to the network. So, in order to get the 
        annotation of a certain individual, you need to access to the individual_dict, which is stored as network attribute.
        >>> print mynet['individuals_dict']['HG3']['continent']
        AFR
        """
        # Check that the genotypes argument contains a list of strings of the same length
#        print "GENOTYPES",  genotypes[0:5]
#        print "INDIVIDUALS", individuals[0:5]
#        print "INDIVIDUALS_DICT", individuals_dict.items()[0:2]

        if not (type(genotypes) in (list, tuple) ) or not (set([len(x) for x in genotypes]) == set([self['chromosome_len']])):
            raise GenotypeNetworkInitError("The list of genotypes provided to .populate_from_binary_strings_and_individuals is incorrect. {0} (network {1})".format(str(set(genotypes)), self['name']))
        
        self['distance_definition'] = distance

        # if there are no information on individuals, this method is more efficient to add nodes.
        if not (len(genotypes) == len(individuals)):
#                logging.debug( genotypes)
#                logging.debug( individuals)
#                logging.debug( individuals_dict.keys())
            raise GenotypeNetworkInitError("individuals var must be of the same length as genotypes")
        
        self['individuals'] = individuals
        self['individuals_dict'] = individuals_dict

        # These lists contain values related to each node. Each element of these lists is relative to a single node.
        node_genotypes = []
        node_n_datapoints = []
        node_individuals = [] 
        node_continent = []         # TODO: clean node_continent, node_subpops, etc...
        node_subpops = []
        node_phenotypes = {}
        phenotypes_keys = [x for x in individuals_dict[individuals_dict.keys()[0]].keys() if x not in ['name', ]]
        allcontinents = []
        self['phenotypes'] = phenotypes_keys
        self['allphenotype_statuses'] = {}
        for phenotype_key in phenotypes_keys:
            node_phenotypes[phenotype_key] = []
            self['allphenotype_statuses'][phenotype_key] = []

        for i in range(len(genotypes)):
            genotype = genotypes[i]
            individual = individuals[i]
            individual_key = individual.replace('_1', '').replace('_2', '')
            continent = individuals_dict[individual_key]['continent']
            if continent not in allcontinents:
                allcontinents.append(continent)
            subpop = individuals_dict[individual_key]['subpop']
#            print phenotypes_keys
#                node_phenotypes[phenotype_key].append(individuals_dict[individual_key][phenotype_key])

            if genotype not in node_genotypes:
                node_genotypes.append(genotype)
                node_n_datapoints.append(1)
                node_individuals.append(set([individual]))
                node_continent.append(set([continent]))
                node_subpops.append(set([subpop]))
#                for phenotype_key in phenotypes_keys:
#                    node_phenotypes[phenotype_key] = [individuals_dict[individual_key][phenotype_key]]
                for phenotype_key in phenotypes_keys:
                    phenotype_status = individuals_dict[individual_key][phenotype_key]
                    node_phenotypes[phenotype_key].append(set([phenotype_status]))
                    if phenotype_status not in self['allphenotype_statuses'][phenotype_key]:
                       self['allphenotype_statuses'][phenotype_key].append(phenotype_status) 
##                node_techniques.append(set([t for t in techniques]))
            else:
                current_node_index = node_genotypes.index(genotype)
                node_n_datapoints[current_node_index] += 1
                node_individuals[current_node_index].add(individual)
                node_continent[current_node_index].add(continent)
                node_subpops[current_node_index].add(subpop)
                for phenotype_key in phenotypes_keys:
                    phenotype_status = individuals_dict[individual_key][phenotype_key]
                    node_phenotypes[phenotype_key][current_node_index].add(phenotype_status)
                    if phenotype_status not in self['allphenotype_statuses'][phenotype_key]:
                       self['allphenotype_statuses'][phenotype_key].append(phenotype_status) 
#                for phenotype_key in phenotypes_keys:
#                    node_phenotypes[phenotype_key].append(individuals_dict[individual_key][phenotype_key])
##                    node_phenotypes[phenotype_key] = individuals_dict[individual_key][phenotype_key]
##                node_techniques[current_node_index].update(set([t for t in techniques]))
#            print genotype, node_phenotypes

        self.add_vertices(len(node_genotypes))
        self['allcontinents'] = allcontinents
        self.vs['genotype'] = node_genotypes
        self.vs['n_datapoints'] = node_n_datapoints
        try:
            self.vs['value'] = [int(n, base=2) for n in node_genotypes]
        except ValueError:
            raise GenotypeNetworkInitError("something went wrong when converting genotypes to binary strings. Check if any SNP is triallelic")
        self.vs['node_individuals'] = [list(n) for n in node_individuals]
        self.vs['continent'] = [list(n) for n in node_continent]
        self.vs['cont_str'] = ['_'.join(conts) for conts in self.vs['continent']]
        self.vs['subpop'] = [list(n) for n in node_subpops]
        self['allphenotypes_statuses'] = {}
        for phenotype_key in phenotypes_keys:
            self.vs[phenotype_key] = [sorted(list(n)) for n in node_phenotypes[phenotype_key]]
#        for phenotype_key in all_phenotypes:


        # Calculate edges
        for genotype1 in self.vs:
#            logging.debug(str((genotype1.index, genotype1['name'], genotype1['value'])))
            for genotype2 in self.vs:
                if genotype1.index >= genotype2.index:
                    pair_distance = '{0:08b}'.format(genotype1['value']^genotype2['value']).count('1')
                    if pair_distance > 0 and pair_distance <= distance:
#                    print genotype1['name'], genotype2['name'],
#                    print '{:08b}'.format(genotype1['value']^genotype2['value']).count('1')
                        self.add_edge(genotype1.index, genotype2.index)

    def subgraph_by_continent(self, continent_name, n_individuals=0, random_seed=False):
        """
        Create a subgraph of the network by extracing all the individuals belonging to a continent

        This is basically a wrapper for __subgraph_by_continent__, without the need for specifying individuals id manually.

        The parameter "continent_name" can take a few special keywords, to indicate a subset of individuals:
        - continent_name == "global_noAMR" -> all individuals, except Native Americans.
        - continent_name == "global_sub"   -> all individuals, but subsampling based on n_individuals.

                                  x             x      x
        >>> genotypes = ['001', '011', '000', '000', '100', '010']
        >>> individuals = ['HG1', 'HG2', 'HG3', 'HG4', 'HG5', 'HG6']
        >>> individuals_dict = {
        ...             'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
        ...             'HG2': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
        ...             'HG3': {'continent': 'EUR', 'subpop': 'AFR', 'phenotype1': 'bad'}, 
        ...             'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
        ...             'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'bad'}, 
        ...             'HG6': {'continent': 'AMR', 'subpop': 'AMR', 'phenotype1': 'good'}, 
        ... }
        >>> mynet = GenotypeNetwork(3, name='mynet')
        >>> mynet.populate_from_binary_strings_and_individuals(genotypes=genotypes, individuals=individuals, individuals_dict=individuals_dict)

        >>> AFR_net = mynet.subgraph_by_continent('AFR')
        >>> print AFR_net
        Genotype Network (name = 'mynet_AFR', chromosome lenght = 3, |V| = 3, |E| = 1)

        >>> AFR_net.vs['genotype']
        ['011', '000', '100']
        >>> AFR_net.vs['node_individuals']
        [['HG2'], ['HG4'], ['HG5']]
        >>> AFR_net.vs['continent']
        [['AFR'], ['AFR', 'EUR'], ['AFR']]
        >>> AFR_net.vs['n_datapoints']
        [1, 1, 1]
        
        Note: if you want to select the nodes belonging to a pop manually, use this list comprehension:
        >>> node_indexes = [v.index for v in mynet.vs if 'AFR' in v['continent']]
        >>> print node_indexes
        [1, 2, 3]

        Test: all network attributes should be the same for a network and its subset 
        >>> sorted(AFR_net.attributes()) == sorted(mynet.attributes())
        True

        Test: the same as above, for vertices
        >>> mynet.vertex_attributes() == AFR_net.vertex_attributes()
        True

        Test: select only 2 individuals
        >>> subgraph = mynet.subgraph_by_continent('AFR', random_seed=2, n_individuals=2)
        >>> print [v['genotype'] for v in subgraph.vs()]
        ['011', '000']
        >>> print [v['node_individuals'] for v in subgraph.vs()]
        [['HG2'], ['HG4']]
        >>> print [v['continent'] for v in subgraph.vs()]
        [['AFR'], ['AFR', 'EUR']]

        >>> subgraph = mynet.subgraph_by_continent('AFR', random_seed=10, n_individuals=2)
        >>> print [v['genotype'] for v in subgraph.vs()]
        ['011', '100']


        # Special keywords
        >>> subgraph_AMR = mynet.subgraph_by_continent('AMR')
        >>> print [v['genotype'] for v in subgraph_AMR.vs()]
        ['010']

#        >>> subgraph_AMR = mynet.subgraph_by_continent('global_noAMR')  # global_noAMR keyword not supported in vcf2networks
#        >>> print [v['genotype'] for v in subgraph_AMR.vs()]
#        ['001', '011', '000', '100']
#        >>> subgraph_AMR = mynet.subgraph_by_continent('global_noAMR', random_seed=10, n_individuals=2)
#        >>> print [v['genotype'] for v in subgraph_AMR.vs()]
#        ['001', '100']
        >>> subgraph_AMR = mynet.subgraph_by_continent('global_sub', random_seed=10, n_individuals=2)
        >>> print [v['genotype'] for v in subgraph_AMR.vs()]
        ['011', '010']
        >>> subgraph_AMR = mynet.subgraph_by_continent('global_sub', random_seed=10, n_individuals=30)
        >>> print subgraph_AMR
        None
        """
        subgraph = self.subgraph_by_phenotype('continent', continent_name, n_individuals=n_individuals, random_seed=random_seed)
        return subgraph

    def __subgraph_by_phenotype__(self, nodes, phenotype, phenotype_status, selected_individuals=False):
        """
        subgraph a network by a given phenotype, keeping only the individuals having a given phenotype_status. 
        
        This method is called internally by .subgraph_by_pop
        
        This is designed to subgraph networks that have information on 
            individuals; for subgraphing a normal network, 
            it would be better to use the standard 
            igraph.Graph.subgraph function.
        """
        subgraph = igraph.Graph.subgraph(self, nodes)
#        subgraph.layout_initialized = False
        subgraph.network_properties_calculated = False

        subgraph['name'] = self['name'] + '_' + phenotype_status
#        subgraph['pop'] = continent
#        subgraph['continent'] = continent
        subgraph[phenotype] = phenotype_status
#        print type(subgraph['individuals_dict'])
#        print subgraph['individuals_dict'][0:100]
#        print subgraph['individuals_dict'][-100:]
#        print ast.literal_eval(subgraph['individuals_dict'])
        if isinstance(subgraph['individuals_dict'], str):
#            print subgraph['individuals_dict']
            subgraph['individuals_dict'] = ast.literal_eval(subgraph['individuals_dict'])
#        print type(subgraph['individuals_dict'])
        if selected_individuals is False:
            subgraph_individuals = [ind.replace("'", '') for ind in subgraph['individuals'] if subgraph['individuals_dict'][ind.replace("'", '').replace('_1', '').replace('_2', '')][phenotype] == phenotype_status]
        else:
            subgraph_individuals = [ind.replace("'", '') for ind in selected_individuals]
        subgraph['individuals'] = subgraph_individuals 

        # http://code.activestate.com/recipes/115417-subset-of-a-dictionary/
        subgraph['individuals_dict'] = dict((ind, subgraph['individuals_dict'][ind]) for ind in subgraph_individuals if ind in subgraph['individuals_dict'])
        for node in subgraph.vs:
#            node['node_individuals'] = [ind for ind in node['node_individuals'] if ind in subgraph_individuals]
            node['node_individuals'] = [ind for ind in node['node_individuals'] if ind in subgraph_individuals]
#            print "genotype", node['genotype'], "INds", node['node_individuals']
            node['n_datapoints'] = len(node['node_individuals'])
        return subgraph


    def subgraph_by_phenotype(self, phenotype, phenotype_status, n_individuals=0, random_seed=False):
        """
        Create a subgraph of the network by extracing all the individuals belonging having a given phenotype status

        This is basically a wrapper for __subgraph_by_phenotype__, without the need for specifying individuals id manually.

#        The parameter "continent_name" can take a few special keywords, to indicate a subset of individuals:
#        - continent_name == "global_noAMR" -> all individuals, except Native Americans.
#        - continent_name == "global_sub"   -> all individuals, but subsampling based on n_individuals.

                                  x             x      x
        >>> genotypes = ['001', '011', '000', '000', '100', '010']
        >>> individuals = ['HG1', 'HG2', 'HG3', 'HG4', 'HG5', 'HG6']
        >>> individuals_dict = {
        ...             'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'happy'}, 
        ...             'HG2': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'sad'}, 
        ...             'HG3': {'continent': 'EUR', 'subpop': 'AFR', 'phenotype1': 'happy'}, 
        ...             'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'sad'}, 
        ...             'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'happy'}, 
        ...             'HG6': {'continent': 'AMR', 'subpop': 'AMR', 'phenotype1': 'happy'}, 
        ... }
        >>> mynet = GenotypeNetwork(3, name='mynet')
        >>> mynet.populate_from_binary_strings_and_individuals(genotypes=genotypes, individuals=individuals, individuals_dict=individuals_dict)

        >>> print mynet.vs['phenotype1']
        [['happy'], ['sad'], ['happy', 'sad'], ['happy'], ['happy']]

        >>> happy_net = mynet.subgraph_by_phenotype('phenotype1', 'happy')
        >>> print happy_net
        Genotype Network (name = 'mynet_happy', chromosome lenght = 3, |V| = 4, |E| = 3)

        >>> happy_net.vs['genotype']
        ['001', '000', '100', '010']
        >>> happy_net.vs['node_individuals']
        [['HG1'], ['HG3'], ['HG5'], ['HG6']]
        >>> happy_net.vs['phenotype1']
        [['happy'], ['happy', 'sad'], ['happy'], ['happy']]
        >>> happy_net.vs['n_datapoints']
        [1, 1, 1, 1]
        >>> happy_net.vs['continent']
        [['EUR'], ['AFR', 'EUR'], ['AFR'], ['AMR']]

        >>> AFR_net = mynet.subgraph_by_phenotype('continent', 'AFR')
        >>> print AFR_net
        Genotype Network (name = 'mynet_AFR', chromosome lenght = 3, |V| = 3, |E| = 1)

        >>> AFR_net.vs['genotype']
        ['011', '000', '100']
        >>> AFR_net.vs['node_individuals']
        [['HG2'], ['HG4'], ['HG5']]
        >>> AFR_net.vs['continent']
        [['AFR'], ['AFR', 'EUR'], ['AFR']]
        >>> AFR_net.vs['n_datapoints']
        [1, 1, 1]
        
        >>> subgraph = mynet.subgraph_by_phenotype('phenotype1', 'happy', random_seed=10, n_individuals=2)
        >>> print [v['genotype'] for v in subgraph.vs()]
        ['001', '010']


        """
#        print [v.attributes() for v in self.vs if 'EUR' in v['continent'] or 'AFR' in v['continent'] or 'ASN' in v['continent']]
        if n_individuals == 0:
            # If n_individuals is 0, select all the individuals from the population.
#            print [v[phenotype] for v in self.vs]
            nodes = [v.index for v in self.vs if phenotype_status in v[phenotype]]
            selected_individuals = False
#            print continent_name, selected_individuals
        else:
            selected_individuals = []
            phenotype_list = []
#            print self[phenotype]
            if phenotype_status == 'global_sub':
                phenotype_list = self['allphenotype_statuses'][phenotype] 
            else: 
                phenotype_list = [phenotype_status]
#            print self['allphenotype_statuses'][phenotype]
#            print phenotype_list
            for ind_key in self['individuals']:
                ind = ind_key.replace('_1', '').replace('_2', '')
                if self['individuals_dict'][ind][phenotype] in phenotype_list:
                    selected_individuals.append(ind_key)


            # subsampling individuals
            if len(selected_individuals) < n_individuals:
                logging.warn(
                    "Not enough individuals in the population to sample (pop: {0}; total individuals: {1}; individuals requested: {2})".format(
                        phenotype_status, len(selected_individuals), n_individuals)
                    )
                return None
            else:
                if random_seed:
                    random.seed(random_seed)
#                random.seed(2)
                random.shuffle(selected_individuals)
                selected_individuals = selected_individuals[:n_individuals]
                nodes = [[v.index for v in self.vs() if ind in v['node_individuals']][0] for ind in selected_individuals]
            
#            print selected_individuals[:4]
#        print pop_individuals
        subgraph = self.__subgraph_by_phenotype__(nodes, phenotype, phenotype_status, selected_individuals=selected_individuals)
        return subgraph

    def csv_report(self, report_attributes = ["name", "continent", "n_snps", "n_genotypes", "n_vertices", "n_edges", "n_components",
                             "diameter", "density", "av_datapoints_per_node", "av_path_length", "av_degree", "max_degree",
                            "av_betweenness", "av_closeness"
                            ]
                       ):
        """
        Print summary statistics for a genotype network

        Example:
        >>> mynet = GenotypeNetwork(chromosome_len=3,  name="Mygene", genotypes=["000", "100","001", "010", "111"])
        >>> print mynet.csv_report()
        Mygene glob 3 5 5 3 2 2 0.3 1.0 1.5 1.2 3 0.6 0.38
        """
#    print network.summary()
        self.__calculate_network_properties__()
#        logging.debug ([(att, att in self.attributes()) for att in report_attributes])
        report_values = []
        for att in report_attributes:
            att = att.lower()
            if att not in self.attributes():
#                print '\n - '.join(self.attributes())
                raise InvalidNetworkAttribute('Invalid attribute: {0}'.format(att), self.attributes())
            else:
                report_values.append(str(self[att]))
#        report_values = [str(self[att]) for att in report_attributes]
        report = ' '.join(report_values)
        return report


    def values_by_node(self, report_attributes=None, header=True):
        """
        Print all the network values, one line per node.

        This function is useful if you want to plot histograms or bloxpot of the 
            property distributions for a network.
        
        >>> phenotype1 = GenotypeNetwork(3, name='phenotype1', genotypes=['011', '010', '000'])
        >>> print (phenotype1.values_by_node()) #doctest: +NORMALIZE_WHITESPACE
        #name       genotype   degree  betweenness  closeness
        phenotype1  010        2       1.0          1.0 
        phenotype1  011        1       0.0          0.6667  
        phenotype1  000        1       0.0          0.6667  

        >>> print (phenotype1.values_by_node(report_attributes=['genotype', 'degree'])) #doctest: +NORMALIZE_WHITESPACE
        #genotype   degree
        010 2   
        011 1   
        000 1   

        """
#        self.__calculate_network_properties__()

        if report_attributes is None:
            report_attributes = ['name', 'genotype', 'degree', 'betweenness', 'closeness']
        report_attributes = [att.lower() for att in report_attributes]

        if header is True:
            report = "#" + "\t".join(report_attributes) + '\n'
        else:
            report = ""
        for node in self.vs:
            node_id = node.index
            node_attributes = []

            for attribute in report_attributes:
                if attribute in self.vertex_attributes():
                    node_attributes.append(str(round_if_float(node[attribute])))
#                    print node[attribute],
                elif hasattr(self, attribute):
#                elif attribute in ("n_edges", ):
#                    print(attribute, eval("self."+attribute+"()"))
                    node_attributes.append(str(round_if_float(eval("self."+attribute+"()")[node_id])))
#                    print 
                elif attribute in self.attributes():
                    # if the attribute is not relative to the node, look for it 
                    #   in the network attributes.
                    node_attributes.append(str(round_if_float(self[attribute])))
                elif attribute.startswith('network_') and attribute[8:] in self.attributes():
                    # if the attribute name starts with network_ (e.g. network_name), get the network property.
                    node_attributes.append(str(round_if_float(self[attribute[8:]])))
                else:
                    raise InvalidNetworkAttribute('attribute "{0}" not found'.format(attribute), self.attributes())
            report += '\t'.join(node_attributes) + '\n'
#            print report_values            
        return report[:-1]

    def plot(self, **args):
        """
        Utility function to plot the Genotype Network.

        This function is a wrapper of the igraph.plot function, except that it plots 
          the network using a standard layout, facilitating the comparison of different networks.

        It accepts all the parameters of igraph.plot, except that some attributes have default values.

        Let's make an example of Genotype Network:

        #doctest: +SKIP
        >> from src.GenotypeNetwork import *

        #doctest: +SKIP
        >> g = debug_GenotypeNetwork()  # this will give an example of GenotypeNetwork, with 4 nodes and 5 individuals

        Plot the network. By default, the genotypes will be used as node labels: #doctest: +SKIP
        >> g.plot()

        Plot the network, using the individuals names instead of the genotypes:  #doctest: +SKIP
        >> g.plot(
        ...        vertex_label=[",".join(v['node_individuals']) for v in g.vs()])

        Plot the network, using individuals as labels, and the number of individuals as node size: #doctest: +SKIP
        >> g.plot(
        ...         vertex_label=[','.join(v['node_individuals']) for v in g.vs()], 
        ...         vertex_size=[len(v['node_individuals']*10) for v in g.vs(), ]


        To save the plot to a file, just pass the parameter "target" #doctest: +SKIP
        >> g.plot(target="mynet.png")
        

        NOTE: the code to calculate node position is actually very inefficient. Applying it to networks larger than 20 SNPs may cause memory errors.
        In this case, it is recommended to call the igraph.plot function directly, giving it the GenotypeNetwork as first argument: #doctest: +SKIP
        >> igraph.plot(g, 
        ...         vertex_label=[','.join(v['node_individuals']) for v in g.vs()], 
        ...         vertex_size=[len(v['node_individuals']*10) for v in g.vs()]
        """

        args.setdefault("vertex_size", 100/self['chromosome_len'])
        args.setdefault('palette', igraph.palettes['red-blue'])
        args.setdefault("vertex_label", self.vs['genotype'])
        args.setdefault("vertex_label_size", 100/self['chromosome_len'])
        args.setdefault("margins", [50,50,50,50])

        if not 'layout' in args:
            if self.layout_initialized is False:
                self.__get_layout__()
            args['layout'] = self.vs['layout']

        igraph.plot(self, **args)
       
def network_from_graphmlz(graphmlfile_path):
    """
    wrapper for network_from_file
    """
    return network_from_file(graphmlfile_path, fileformat='graphmlz')

def network_from_graphml(graphmlfile_path):
    """
    wrapper for network_from_file
    """
    return network_from_file(graphmlfile_path, fileformat='graphml')

def network_from_file(graphmlfile_path, fileformat='graphml'):
    """
    import a GenotypeNetwork from a graphml file.

    The function igraph.load returns a igraph.Graph object, but I want a GenotypeNetwork object, so I need this wrapper function to reconvert.

    Note that this function would work with any format, not only graphml.

    Note2: the returned network has a new attribute, vs['id'], created when the network is converted to graphml in the first place.
    """
    if fileformat == 'graphml':
        loaded_network = igraph.Graph.Read_GraphML(graphmlfile_path)
    elif fileformat == 'graphmlz':
        loaded_network = igraph.Graph.Read_GraphMLz(graphmlfile_path)
    else:
        loaded_network = igraph.load(graphmlfile_path)
    chromosome_len = int(loaded_network['chromosome_len'])

    network = GenotypeNetwork(int(chromosome_len))
    network.add_vertices(loaded_network.vcount())
    network.add_edges(loaded_network.get_edgelist())
    network.layout_initialized = False
    network.network_properties_calculated = False

#    network['individuals'] = 
    for att in loaded_network.attributes():
        current_att = loaded_network[att]
        if att in ('chromosome_len', 'distance_definition'):
            current_att = int(current_att)
        network[att] = current_att
    network['individuals'] = ast.literal_eval(network['individuals'])
#    print network["positions"]
#    network['positions'] = ast.literal_eval(network['positions'])
#    network['chromosomes'] = ast.literal_eval(network['chromosomes'])
#    logging.debug( igraph.Graph.summary(network))
    network['individuals_dict'] = ast.literal_eval(network['individuals_dict'])
    for v_att in loaded_network.vertex_attributes():
        current_vatt = loaded_network.vs[v_att]
        if v_att in ('n_datapoints', ):
            current_vatt = [int(i) for i in current_vatt]
        network.vs[v_att] = current_vatt
#    print loaded_network.es
    for e_att in loaded_network.edge_attributes():
        network.es[e_att] = loaded_network.es[e_att]
    return network
       
def get_y_position(node):
    """
    Given a node, return its y position

    This code is pretty crappy, and it is not efficient for > 20 nodes.

    Parameters:

        * n = node, as a binary string. Trailing 0s are important, 
            for example: '0010' will give different results than '10'

    >>> get_y_position('001')
    -0.5

    >>> get_y_position('010')
    0.5

    >>> get_y_position('100')
    1.5

    >>> get_y_position('11000')
    5.0

    >>> get_y_position('00011')
    -4.0

    >>> get_y_position('01100')
    1.0

    >>> get_y_position('01010')
    0.0

    """

    one_count = node.count('1')

    all_values = kbits(len(node), one_count)
    nodeset_len = len(all_values) 
#    pos_y = -(all_values.index(node) + nodeset_len/2.)
    pos_y = -all_values.index(node) + nodeset_len/2.
#    l = ['01'] * one_count
#    l.extend(['00']*(len(node)-one_count))
#    logging.debug(l)
#    all_values = [''.join(n) for n in itertools.product(*l)]
#    logging.debug( all_values)
#    pos_y = all_values.index(node)

    return pos_y



def kbits(chromosome_len, one_occurrencies):
    """
    Generate all combinations of genotypes of lenght chromosome_len that have one_occurrencies '1's

    Credits: http://stackoverflow.com/questions/1851134/generate-all-binary-strings-of-length-n-with-k-bits-set
    """
    result = []
    for bits in itertools.combinations(range(chromosome_len), one_occurrencies):
        current_str = ['0'] * chromosome_len
        for bit in bits:
            current_str[bit] = '1'
        result.append(''.join(current_str))
    return result

def find_neighbors(genotype, genotypes):
    """
    Determine which genotypes are neighbors of a given genotype

    This function is not implemented yet, but I may use if for parallel calculations.
    """
    pass

def is_neighbor(genotype1, genotype2):
    """
    Determine if two genotypes are neighbors

    Note: in the end this function is not used, but I will keep it here for the doctests.

    >>> is_neighbor('000', '001')
    True
    >>> is_neighbor('001', '000')
    True
    >>> is_neighbor('000', '000')
    False
    >>> is_neighbor('000', '111')
    False
    >>> is_neighbor('0000', '111')
    False
    >>> is_neighbor('0001', '0010')
    False
    >>> is_neighbor('0000', '10')
    True

    """
    return '{0:08b}'.format(int(genotype1, base=2)^int(genotype2, base=2)).count('1') == 1

def random_genotype_network(chromosome_len, sample_size, max_variation=None, seed=None):
    """
    Generate a random genotype network, for debugging purposes

    Note: this is a quick and dirty function, and some nodes may be duplicated

    >>> net = random_genotype_network(10, 150, 2**10-1, 10)
    >>> print net.summary() #doctest: +NORMALIZE_WHITESPACE
    140 nodes, 96 edges, undirected
    <BLANKLINE>
    Number of components: 50
    Diameter: 13
    Density: 0.0099
    Average path length: 4.0907

    """
    if max_variation is None:
        max_variation = 2 ** chromosome_len - 1
    if seed is not None:
        random.seed(seed)
    genotype_format = "{0:0" + str(chromosome_len) + "b}"
    genotypes = [genotype_format.format(random.randint(0, max_variation)) for i in range(sample_size)]                
    network = GenotypeNetwork(chromosome_len, genotypes=genotypes)
    return(network)

def get_mean(numbers):
    """
    Calculate mean of a list of numbers, as silly python doesn't have this function by default (and I don't want to import numpy)
    >>> get_mean((1, 3))
    2.0
    >>> print get_mean(())
    nan
    """
    if len(numbers) > 0:
        return round((1.0 * sum(numbers) / len(numbers)), 4)
    else:
        return numpy.nan

def round_if_float(number):
    """
    if number is a float, round it to 4 decimals

    >>> round_if_float(0.4) == 0.4
    True
    >>> round_if_float("dsada")
    'dsada'
    >>> round_if_float(0.1234567)
    0.1235
    >>> round_if_float(0)
    0
    """
#    print number, type(number)
    if isinstance(number, float):
        return round(number, 4)
    else:
        return number

def get_max(numbers):
    """
    Wrapper for silly python, that returns an error if the max function is called on a empty list

    >>> get_max([1.0,2.0,3.0])
    3.0
    >>> get_max([1,2,3])
    3
    >>> print get_max([])
    None
    """
    if len(numbers) > 0:
        maxvalue = max(numbers)
        if isinstance(maxvalue, float):
            maxvalue = round(maxvalue, 4)
    else:
        maxvalue = None
    return maxvalue


def exampleNetwork():
    """
    An example Genotype Network, window_size 3, n. individuals 5

    """
    genotypes = ['001', '011', '000', '000', '100']
    individuals = ['HG1', 'HG2', 'HG3', 'HG4', 'HG5']
    individuals_dict = {
         'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
         'HG2': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'bad'}, 
         'HG3': {'continent': 'EUR', 'subpop': 'AFR', 'phenotype1': 'good'}, 
         'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
         'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
         }
    mynet = GenotypeNetwork(3, name='mynet')
    mynet.populate_from_binary_strings_and_individuals(genotypes=genotypes, individuals=individuals, individuals_dict=individuals_dict)
    return mynet

def exampleNetworkBig():
    """
    A more complex example of Genotype Network

    """
    name = "diploid_network"
    chromosome_len = 7
    genotypes =   [
            '0000000',    
            '0001000',   
            '1000000',   
            '0000001',   
            '0000010',   
            '0001010',   
            '1000111',   
            '0111110',   
            '0101110',   
            '0100110',
            '0100011',
            '0100010',
            '0101010',
            '0111010',
            '0000100',
            '0100000',
            '0001010',
            '1010101']
                
    individuals = ['HG' + str(i/2) + '_' + str((i%2)+1) for i in range(len(genotypes))]
    individuals_dict = {
                    'HG0': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                    'HG1': {'continent': 'EUR', 'subpop': 'TSC', 'phenotype1': 'good'}, 
                    'HG2': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'bad'}, 
                    'HG3': {'continent': 'AFR', 'subpop': 'AFR', 'phenotype1': 'bad'}, 
                    'HG4': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                    'HG5': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                    'HG6': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'good'}, 
                    'HG7': {'continent': 'AFR', 'subpop': 'YRI', 'phenotype1': 'bad'}, 
                    'HG8': {'continent': 'EUR', 'subpop': 'GBR', 'phenotype1': 'neutral'}, 
    }

    mynet = GenotypeNetwork(7, name='mynet')
    mynet.populate_from_binary_strings_and_individuals(genotypes=genotypes, individuals=individuals, individuals_dict=individuals_dict)
    return mynet

def plot_ExampleBig():
    g = exampleNetworkBig()
    g.__get_layout__()
    igraph.plot(g, layout=g.vs['layout'], vertex_size=60, vertex_label=g.vs['genotype'], vertex_shape='rectangle')



if __name__ == '__main__':
#    import logging
    import doctest
#    logging.basicConfig(level=logging.DEBUG)
    doctest.testmod()

